PROGRAM dechar_111
!
!***********************************************************
!*                                                         *
!*  DEfect CHARacterization in {111} glide planes          *
!*  ==     ====                 ===                        *
!*                                                         *
!*  Version 0.2e  (still under developpement)              *
!*  (C) Oct. 2005 - P.HIREL                                *
!*  pierre.hirel@etu.univ-poitiers.fr                      *
!*  (Last change: 11/2007 by P. HIREL)                     *
!*                                                         *
!*  This program parses all or selected {111} planes of    *
!*  a "perfect" crystal,  and compares atomic positions    *
!*  with  the  ones  of  a  relaxed  crystal.  Then  it    *
!*  analyzes    relative    displacements    of   atoms    *
!*  in  these   {111}   glide  planes ,   and  compares    *
!*  them  to  a  perfect  60°  dislocation  components.    *
!*  Finally  it  outputs  edge,  screw  and  orthogonal    *
!*  components   in    separate   files ,    for   each    *
!*  "interesting" plane (ie, planes which present a 30°    *
!*  60°  or  90°-like  displacement  on  the  surface).    *
!*  Multiple  files  can  be  analyzed  in  a  run, and    *
!*  dislocation position, speed and core extension  can    *
!*  be output.                                             *
!*  More informations on this program are available in     *
!*  the file 'dechar_111.readme'                           *
!*                                                         *
!*  Input : dechar_111.in                                  *
!*          Two position files (XYZ or CFG format)         *
!*                                                         *
!*  Output : dechar_111.log    (log for this program)      *
!*           dechar_111.nfo    (short info. on progress)   *
!*           <outfile>         (file for final results)    *
!*        Depending on options set in dechar_111.in :      *
!*           edge_allZ_X.dat   (edge components)           *
!*           ddedge_X.dat      (derivative of edge comp.)  *
!*           screw_allZ_X.dat  (screw components)          *
!*           edge_AVG.dat      (average edge comp.)        *
!*           screw_AVG.dat     (average screw comp.)       *
!*           core_X.dat        (core position in plane)    *
!*           nz_X.cfg          (atomic pos. in dz-layer)   *
!*           Vortho_X.dat      (disloc. speed)             *
!*           Vpara_X.dat       (disloc. speed)             *
!*                                                         *
!*  Tested (compiles and works) with                       *
!*  Intel Fortran Compiler (ifort)  and                    *
!*  Portland Group Fortran (pgf95)  compilers.             *
!*  No additionnal library required.                       *
!*                                                         *
!***********************************************************
!*  This program is developped under GNU GPL :             *
!*    http://www.gnu.org/licenses/gpl.html                 *
!*  This source code shall always be available freely.     *
!*                                                         *
!***********************************************************
!
!
IMPLICIT NONE
!
CHARACTER(LEN=1) :: ich1, rich1  ! character strings used to
CHARACTER(LEN=2) :: ich2, rich2  !    store integers
CHARACTER(LEN=2) :: atomsp  ! atom spieces
CHARACTER(LEN=3) :: ich3, rich3  ! idem as ich1 and rich1
CHARACTER(LEN=3) :: cfgAtxt  !temporary text
CHARACTER(LEN=3) :: xyzNRtype, xyzRtype !type of position files : xyz or cfg
CHARACTER(LEN=4) :: rich4  ! idem as ich1 and rich1
CHARACTER(LEN=5) :: rich5  !  ""
CHARACTER(LEN=7) :: datatype !used when parsing dechar_111.in
CHARACTER(LEN=7) :: lattice  ! lattice geometry
CHARACTER(LEN=7) :: diamondgp  ! glide plane in diamond structure
CHARACTER(LEN=8) :: particle, particle1, particle2   !Reading xMD position file
CHARACTER(LEN=9) :: mattxt !matrix text 'H0(n,n) ='
CHARACTER(LEN=21) :: particlecfg   ! Reading CFG position file
CHARACTER(LEN=32) :: cfgtemp  !Reading irrelevant information in CFG files
CHARACTER(LEN=32) :: CwidthT  ! File name for core width vs time
CHARACTER(Len=32) :: infile   ! Name of input file
CHARACTER(LEN=32) :: coredatfile, corewidthdatfile           ! Indexed
CHARACTER(LEN=32) :: ddedgefile, edatfile, edgfile           ! output
CHARACTER(LEN=32) :: efitfile, odatfile, scrfile, sdatfile   ! files
CHARACTER(LEN=32) :: partcfgout    ! name of the output position file for dz
CHARACTER(LEN=32) :: eavgfile, savgfile !output files for averaged values
CHARACTER(LEN=32) :: recfileprefix, recfilesuffix !used in recursive mode
CHARACTER(LEN=32) :: VorthoT, VparaT, VorthoS
CHARACTER(LEN=64) :: outfile, xyzNR, xyzR, currfile  !file names
INTEGER :: addyl, addyr  ! planes added on the left/on the right
INTEGER :: coredone ! checks if dislocation core is detected
INTEGER :: corefrac
INTEGER :: counterror, countwarning
INTEGER :: datado, donealloc, donefinal ! checkers for input file
INTEGER :: donecrystal, donefilenam, donesdiscar ! checkers for input file
INTEGER :: donerecanal  ! checkers for input file
INTEGER :: donedisfront, disfront  !Detection of dislocation front
INTEGER :: docfgdz  ! trigger to output atomic positions for analyzed dz
INTEGER :: forceoutput  !if 1 output of .dat files is forced
INTEGER :: gnuplot !flag: 1=use gnuplot; 0=dont use
INTEGER :: HLCS  ! Half-Loop Critical Size
INTEGER :: i, j, h, k, l, n, m, p, ii, i2, jj, natom, natom1, natom2 !loops
INTEGER :: icore, itruc, idecmax, icol, icz, it, nn, pllbl
INTEGER :: idec, sidec ! X-position of {111} plane relative to the origin
INTEGER :: idz, sidz   ! Label of the Z-layer relative to origin (Z=0)
INTEGER :: idzvm5, idzvm2, idzvm1, idzvp1, idzvp2, idzvp5  !used for disloc velocity
INTEGER :: intplane, intplaneup, intplindex, isintplane
                    ! Indexes interesting {111} planes
INTEGER :: iplane, iplanemax
INTEGER :: natcmax, natcup, natpmax, natcmin, nidz, nzcfgcount, mm, mm2
INTEGER :: nx, ny, nz  ! number of planes in X, Y and Z directions
INTEGER :: nbintpl, idecorig
INTEGER :: orxy, orxz, oryz
INTEGER :: outedgef, outscrewf, outorthof            ! Checks which files are
INTEGER :: outcoreposf, outcorewidthf, outvelocityf  ! to be output
INTEGER :: recanal, recindex, recindexmax, reccount, vcount, vcountmax !recursive analysis
INTEGER :: rnstart, rnend, rninc !Starting and ending values of N for recursive analysis
INTEGER :: rninc1, rninc2, rnincmem !auto-detection of rninc
INTEGER :: slopeid, sslopeid  ! Sign of the slope of the {111} planes parsed
INTEGER :: outofbox, warning, warning2 ! used for warning messages
INTEGER :: write22 ! checks whenever data is written in file 22
                   ! (ie, output file)
INTEGER :: writelog
REAL*8 :: a0  ! Lattice parameter
REAL*8 :: allon, fish  ! % deformation and Poisson's ratio
REAL*8 :: atommass !mass of atoms
REAL*8 :: ax, ay, az, axs2, bedge, bscrew !base and Burgers vectors
REAL*8 :: cfgA, cfgx, cfgy, cfgz  ! Reading reduced coordinates in CFG files
REAL*8 :: coreav ! average position of the core in a {111} plane
REAL*8 :: corewidth1, corewidth2
REAL*8 :: co, delta, pa, u, u0, si  !trigo
REAL*8 :: d111  !distance between {111} planes
REAL*8 :: dedgetot, dscrewtot, dorthotot !total components
REAL*8 :: dis, dis1, dis2, dis5, dis11, dis12, dis15  ! Final valor of disp.
REAL*8 :: distmin, distmin2  ! Distances relative to origin
REAL*8 :: dtime ! Time step used in simulation (ps)
REAL*8 :: etime, elapsed(2), total  !To measure running time of program
REAL*8 :: fita, fitb, fitc, fitd  !fitted values (core position)
REAL*8 :: gnuplotfit  !temporary core position from GnuPlot fit
REAL*8 :: htz  ! Total Z width of the crystal
REAL*8 :: maxddedge !max. of derivative of edge disp. = disloc.core position
REAL*8 :: mvalue  ! Reading values of transformation matrix H0 in CFG files
REAL*8 :: nucleationtime ! Detection of the nucleation instant
REAL*8 :: pente, pinv  ! slope
REAL*8 :: prox  ! Distance from chosen (111) plane
REAL*8 :: tr1, tr2 ! X and Y coordinates of the mm-th atom
REAL*8 :: x02, xidz
REAL*8 :: xvar
REAL*8 :: xmin, xmax, ymin, ymax, zmin, zmax, zmin0, zmax0, x0, y0
REAL*8 :: xminNR, yminNR, zminNR  !minima (unrelaxed crystal)
REAL*8 :: xmaxNR, ymaxNR, zmaxNR  !maxima (unrelaxed crystal)
REAL*8 :: xminR, yminR, zminR     !minima (relaxed crystal)
REAL*8 :: zmaxR, xmaxR, ymaxR     !maxima (relaxed crystal)
LOGICAL :: exrecfile, inexist  ! Check existence of file
LOGICAL :: done22idec
LOGICAL :: guessrninc, outofboxNR
INTEGER, DIMENSION(2) :: natp, natc, s !number of atoms in plane/Z-layer
INTEGER, DIMENSION(3) :: orx, ory, orz !crystallographic directions along X,Y,Z
INTEGER, ALLOCATABLE, DIMENSION(:,:) :: iatom ! indexing atoms
INTEGER, ALLOCATABLE, DIMENSION(:,:) :: ir, ir1 
REAL*8, DIMENSION(2) :: p1, p2, xdec  ! indexing planes
REAL*8, ALLOCATABLE, DIMENSION(:) :: corewidthT
REAL*8, ALLOCATABLE, DIMENSION(:) :: DPparaL, DPparaR, DPortho
REAL*8, ALLOCATABLE, DIMENSION(:) :: dist ! displacement components
REAL*8, ALLOCATABLE, DIMENSION(:) :: dedgeavg, dscrewavg ! average components
REAL*8, ALLOCATABLE, DIMENSION(:) :: Vortho, Vpara
REAL*8, DIMENSION(3,3) :: H0  !Transformation matrix in CFG files
REAL*8, ALLOCATABLE, DIMENSION(:,:) :: coordNR, coordR ! atomic coordinates
REAL*8, ALLOCATABLE, DIMENSION(:,:) :: coremem ! core position for idz=1,nz
                                               ! in a (111) plane
REAL*8, ALLOCATABLE, DIMENSION(:,:) :: corewidth, gpcorewidth
REAL*8, ALLOCATABLE, DIMENSION(:,:) :: ddep, ddep2, d  ! displacements
REAL*8, ALLOCATABLE, DIMENSION(:,:) :: dscrew, dedge, dortho
                                       ! displacements components
REAL*8, ALLOCATABLE, DIMENSION(:,:) :: yc, yd
REAL*8, ALLOCATABLE, DIMENSION(:,:,:) :: dedgefinal, dscrewfinal, dorthofinal
REAL*8, ALLOCATABLE, DIMENSION(:,:,:) :: ddedge !derivative of edge comp.
REAL*8, ALLOCATABLE, DIMENSION(:,:,:) :: edgefit
REAL*8, ALLOCATABLE, DIMENSION(:,:,:) :: cx, cry, crx0, nzcfgcoord, px
!
WRITE(*,*) 'Starting program...'
WRITE(*,*) ' '
WRITE(*,*) "   ----================================----        __"
WRITE(*,*) "                  DeChar_111                     .'  `. "
WRITE(*,*) "   DEfect CHARacterization in {111} planes       |a_a  |"
WRITE(*,*) "   ----================================----      \<_)__/"
WRITE(*,*) "           This is GNU/GPL software              /(   )\"
WRITE(*,*) "   Feel free to modify and distribute ! -_o     |\`>  /V\"
WRITE(*,*) "   ----============ v.0.2e ============----     \_|=='|_/ "
WRITE(*,*) ' '
!
!
! Initialize variables
sslopeid = 0
slopeid = 0
sidec = 0
sidz = 0
counterror = 0
countwarning = 0
donefilenam = 0
donecrystal = 0
donedisfront = 0
donesdiscar = 0
donefinal = 0
donerecanal = 0
done22idec = .FALSE.
dtime = 4.d-3  !Time step (ps) in simulation (used to calculate disloc. velocities)
forceoutput = 0
gnuplot = 0    !By default, GnuPlot is not used
gnuplotfit = 0.d0
guessrninc = .FALSE.
HLCS = 0
disfront = 0
nucleationtime = 0.d0
orx(:) = 0.d0
ory(:) = 0.d0
orz(:) = 0.d0
outofboxNR = .FALSE.
outedgef = 1
outscrewf = 1
outorthof =1
outcoreposf = 1
outcorewidthf = 1
outvelocityf = 1
recanal = 0
reccount = 1
recindex = 0
recindexmax = 0
rnstart = 0
rnend = 0
rninc = 1
rninc1 = 1
rninc2 = 1
rnincmem = 1
outofbox = 0
vcount = 0
warning = 0
warning2 = 0
write22 = 0
writelog = 0
infile = ''
!
! Check if user specified command-line parameters
! The only command-line parameter allowed for now is
! an alternate input file name
CALL GETARG(1,infile)
! If no file name is specified, use default input file name
IF(LEN_TRIM(infile)==0) infile = 'dechar_111.in'
!
! All informations will be output in a log file (dechar_111.log)
! during the execution of the program
OPEN(21,FILE='dechar_111.log',FORM='FORMATTED',STATUS='UNKNOWN')
WRITE(21,*) "           \\\\\\\\\\\\\\\X///////////////"
WRITE(21,*) " \\\\\\\\\\\\\\\\\\\\\\\\\X/////////////////////////"
WRITE(21,*) " \   dechar_111 :                                  /"
WRITE(21,*) " \   DEfect CHARacterization in {111} planes       /"
WRITE(21,*) " \   ==     ====                 ===               /"
WRITE(21,*) " \ v.0.2e                   (C) Oct.2005 - P.HIREL /"
WRITE(21,*) " \               pierre.hirel@etu.univ-poitiers.fr /"
WRITE(21,*) " \- - - - - - - - - - - - - - - - - - - - - - - - -/"
WRITE(21,*) " \ Program developped under General Public Licence /"
WRITE(21,*) " \                    G P L                        /"
WRITE(21,*) " \      http://www.gnu.org/licenses/gpl.html       /"
WRITE(21,*) " \- - - - - - - - - - - - - - - - - - - - - - - - -/"
WRITE(21,*) " \\\\\\\\\\\\\\\\\\\\\\\\\X/////////////////////////"
WRITE(21,*) ' '
WRITE(21,*) ' NB: This is the log file generated by the program,'
WRITE(21,*) '     mainly used for debugging purposes.'
WRITE(21,*) '     User shall not be able to view it, except if'
WRITE(21,*) "     WRITLOG 1  was set in the input file"
WRITE(21,*) '     or the program unexpectedly crashed before its end.'
WRITE(21,*) ' '
!
!Check if intput file 'dechar_111.in' exists, and if not, create one
INQUIRE(FILE=infile,EXIST=inexist)
IF(.NOT. inexist) THEN
  WRITE(*,*) " (°<  ERROR: This input file was not found: ", infile
  WRITE(21,*) " (°<  ERROR: This input file was not found: ", infile
  OPEN(11,FILE=infile,ERR=11,FORM='FORMATTED',STATUS='NEW')
  WRITE(11,'(a61)') 'FILENAM  1            ! Indicates that file names will follow'
  WRITE(11,'(a74)') "Unrelaxed.xyz         ! File name for unrelaxed crystal's atomic positions"
  WRITE(11,'(a72)') "Relaxed.xyz           ! File name for relaxed crystal's atomic positions"
  WRITE(11,'(a48)') 'MyResults.out         ! Desired output file name'
  WRITE(11,*) ' '
  WRITE(11,'(a69)') 'CRYSTAL  1            ! Indicates that crystal properties will follow'
  WRITE(11,'(a66)') "fcc                   ! Crystal geometry (only 'fcc' is supported)"
  WRITE(11,'(a53)') '4.0200                ! Lattice parameter (Angstroms)'
  WRITE(11,'(a47)') '0 -1 1                ! X vector (ax=Xvector/2)'
  WRITE(11,'(a47)') '1 0 0                 ! Y vector (ay=Yvector/2)'
  WRITE(11,'(a47)') '0 1 1                 ! Z vector (az=Zvector/2)'
  WRITE(11,'(a36)') "Al                    ! Atom spieces"
  WRITE(11,'(a45)') '6.5000                ! % deformation along X'
  WRITE(11,'(a39)') "0.3500                ! Poisson's ratio"
  WRITE(11,'(a72)') '60                    ! Total number of layers orthogonal to X direction'
  WRITE(11,'(a72)') '40                    ! Total number of layers orthogonal to Y direction'
  WRITE(11,'(a72)') '60                    ! Total number of layers orthogonal to Z direction'
  WRITE(11,*) ' '
  WRITE(11,'(a68)') 'SDISCAR  0            ! 1=restrain analysis; 0=analyse and output all'
  WRITE(11,'(a72)') '0                     ! 1=force output; 0=auto-detect interesting planes'
  WRITE(11,'(a49)') '0                     ! 1=use GnuPlot; 0=dont use'
  WRITE(11,'(a73)') '0                     ! slopeid (1=positive; 2=negative; else=parse both)'
  WRITE(11,'(a76)') '0                     ! idec (0=parse all {111} planes with indicated slope)'
  WRITE(11,'(a70)') '0                     ! idz  (0=parse all Z-layers in indicated plane)'
  WRITE(11,'(a53)') "shuffle               ! Glide plane (not used if fcc)"
  WRITE(11,'(a76)') "1 1 1                 ! Output edge, screw, ortho displacements (1=yes,0=no)"
  WRITE(11,'(a76)') "1 1                   ! Output dislocation core position, width (1=yes,0=no)"
  WRITE(11,'(a66)') "1                     ! Output dislocation velocities (1=yes,0=no)"
  WRITE(11,*) ' '
  WRITE(11,'(a74)') 'RECANAL  0            ! 1=recursive analysis on multiple files; 0=disabled'
  WRITE(11,'(a50)') 'Prefix_               ! Prefix of files to analyze'
  WRITE(11,'(a50)') '_Suffix.cfg           ! Suffix of files to analyze'
  WRITE(11,'(a43)') '0                     ! Starting value of N'
  WRITE(11,'(a41)') '0                     ! Ending value of N'
  WRITE(11,'(a39)') '1                     ! Increment for N'
  WRITE(11,'(a56)') '4.e-3                 ! Time step between two files (ps)'
  WRITE(11,'(a80)') ' '
  WRITE(11,'(a72)') 'ENDFILE  1            ! Force end of file (following lines wont be read)'
  WRITE(11,*) ' '
  WRITE(11,'(a33)') 'INPUT FILE FOR dechar_111 PROGRAM'
  WRITE(11,'(a34)') 'Commentaries can be inserted here.'
  WRITE(11,'(a53)') "Refer to the 'dechar_111.readme' file for information"
  WRITE(11,'(a36)') 'on how to configure this input file.'
  CLOSE(11)
  WRITE(21,*) " >>>  The program has created a dummy input file for you: ", infile
  WRITE(21,*) '      Please edit this file according to your system properties'
  WRITE(21,*) '      and run the  dechar_111  program once again.'
  WRITE(*,*) " >>>  The program has created a dummy input file for you: ", infile
  WRITE(*,*) '      Please edit this file according to your system properties'
  WRITE(*,*) '      and run the  dechar_111  program once again.'
  counterror = counterror+1
  GOTO 1100
ENDIF
!
! Reading input file
OPEN(11,FILE=infile,ERR=11,FORM='FORMATTED',STATUS='OLD')
GOTO 12
11 WRITE(*,*) " (°<  ERROR: Unable to find input file: ", infile
   WRITE(*,*) ' (°<         Program will EXIT.'
   WRITE(21,*) " (°<  ERROR: Unable to find input file: ", infile
   WRITE(21,*) ' (°<         Program will EXIT.'
   counterror = counterror+1
   GOTO 1100
!
12 CONTINUE
!
IF((donefilenam==1) .AND. (donecrystal==1) .AND. (donesdiscar==1)       &
  & .AND. (donerecanal==1)) THEN
  WRITE(21,*) ' '
  WRITE(21,*) ' >>>  Done reading intput file: ', infile
  GOTO 20
ENDIF
READ(11,*,ERR=12,END=20) datatype, datado
IF(  (datatype.NE.'FILENAM') .AND. (datatype.NE.'CRYSTAL') .AND.        &
    &(datatype.NE.'FILNAME') .AND. (datatype.NE.'SDISCAR') .AND.        &
    &(datatype.NE.'ENDFILE') .AND. (datatype.NE.'RECANAL') .AND.        &
    &(datatype.NE.'WRITLOG')                                     ) THEN
  GOTO 12
ELSEIF(datatype=='WRITLOG') THEN
  IF(datado==1) writelog=1
  GOTO 12
ELSEIF(datatype=='FILENAM' .OR. datatype=='FILNAME') THEN
  IF(donefilenam==1) GOTO 12
  IF(datado==1) THEN
    donefilenam = 1
    GOTO 15
  ELSE
    GOTO 12
  ENDIF
ELSEIF(datatype=='CRYSTAL') THEN
  IF(donecrystal==1) GOTO 12
  IF(datado==1) THEN
    donecrystal = 1
    GOTO 16
  ELSE
    GOTO 12
  ENDIF
ELSEIF(datatype=='SDISCAR') THEN
  IF(donesdiscar==1) GOTO 12
  IF(datado==1) THEN
    donesdiscar = 1
    GOTO 17
  ELSE
    GOTO 12
  ENDIF
ELSEIF(datatype=='RECANAL') THEN
  IF(donerecanal==1) GOTO 12
  IF(datado==1) THEN
    donerecanal = 1
    GOTO 18
  ELSE
    GOTO 12
  ENDIF
ELSEIF(datatype=='ENDFILE') THEN
  GOTO 20
ELSE
  GOTO 12
ENDIF
!
!
15 CONTINUE
WRITE(21,*) ' '
WRITE(21,*) "PROGRAM'S INPUT FILES NAMES:"
WRITE(21,*) "  Read program's control file: ", infile
! Name of xMD coordinates file for unrelaxed crystal
READ(11,*,ERR=33) xyzNR
currfile = xyzNR
INQUIRE(FILE=xyzNR,EXIST=exrecfile)
IF(.NOT.exrecfile) THEN
  WRITE(*,*) " (°<  ERROR: File '"//TRIM(xyzNR)//"' does not exist."
  WRITE(*,*) " (°<         Please check your 'dechar_111.in' file."
  WRITE(*,*) ' >>>  Program EXIT.'
  WRITE(21,*) " (°<  ERROR: File '"//TRIM(xyzNR)//"' does not exist."
  WRITE(21,*) " (°<         Please check your 'dechar_111.in' file."
  WRITE(21,*) ' >>>  Program EXIT.'
  counterror = counterror+1
  GOTO 1100
ENDIF
WRITE(21,*) '  Read unrelaxed crystal atomic positions: ', xyzNR
! Name of xMD coordinates file for relaxed crystal
READ(11,*,ERR=33) xyzR
currfile = xyzR
INQUIRE(FILE=xyzNR,EXIST=exrecfile)
IF(.NOT.exrecfile) THEN
  WRITE(*,*) " (°<  ERROR: File '"//TRIM(xyzR)//"' does not exist."
  WRITE(*,*) " (°<         Please check your input file: ", infile
  WRITE(*,*) ' >>>  Program EXIT.'
  WRITE(21,*) " (°<  ERROR: File '"//TRIM(xyzR)//"' does not exist."
  WRITE(21,*) " (°<         Please check your input file: ", infile
  WRITE(21,*) ' >>>  Program EXIT.'
  counterror = counterror+1
  GOTO 1100
ENDIF
WRITE(21,*) '  Read relaxed crystal atomic positions: ', xyzR
WRITE(21,*) ' '
WRITE(21,*) "PROGRAM'S OUTPUT FILES NAMES:"
READ(11,*,ERR=33) outfile     ! Desired name for this program's output file
WRITE(21,*) '  Major results will be written in file: ', outfile
WRITE(21,*) "  Output numerical data will be written in [.dat] files."
OPEN(22,FILE=outfile,STATUS='UNKNOWN')
WRITE(22,*) " #  dechar_111:"
WRITE(22,*) " #  DEfect CHARacterization in {111} planes"
WRITE(22,*) " #  --     ----                 ---"
WRITE(22,*) " #  (C) Oct.2005 - P.HIREL"
WRITE(22,*) " #  pierre.hirel@etu.univ-poitiers.fr"
WRITE(22,*) " # "
WRITE(22,*) " ################### v.0.2e ###################"
WRITE(22,*) ' '
WRITE(22,*) 'Summary of analysis:'
WRITE(22,*) '--------------------'
GOTO 12
!
16 CONTINUE
WRITE(21,*) ' '
WRITE(21,*) 'CRYSTAL PROPERTIES:'
! Read lattice geometry (only fcc implemented yet)
READ(11,*,ERR=22) lattice
IF(TRIM(lattice)=='diamond') THEN
  WRITE(*,*) ' (°<  Diamond structure is not implemented yet.'
  GOTO 1000
ELSEIF(TRIM(lattice) .NE. 'fcc') THEN
  GOTO 22
ENDIF
WRITE(21,*) '  Lattice geometry: ', lattice
! Read lattice parameter
READ(11,*,ERR=22) a0
WRITE(21,'(a26,f12.6)') '   Lattice parameter (A): ', a0
! Read lattice orientations
WRITE(21,*) '  Crystallographic orientations:'
READ(11,*,ERR=22) orx(1), orx(2), orx(3)
WRITE(21,'(a9,3i3,a1)') '    X = [', orx(1), orx(2), orx(3), ']'
READ(11,*,ERR=22) ory(1), ory(2), ory(3)
WRITE(21,'(a9,3i3,a1)') '    Y = [', ory(1), ory(2), ory(3), ']'
READ(11,*,ERR=22) orz(1), orz(2), orz(3)
WRITE(21,'(a9,3i3,a1)') '    Z = [', orz(1), orz(2), orz(3), ']'
! Check orthogonality :
orxy = orx(1)*ory(1) + orx(2)*ory(2) + orx(3)*ory(3)
orxz = orx(1)*orz(1) + orx(2)*orz(2) + orx(3)*orz(3)
oryz = ory(1)*orz(1) + ory(2)*orz(2) + ory(3)*orz(3)
IF(orxy.NE.0.d0 .OR. orxz.NE.0.d0 .OR. oryz.NE.0.d0) THEN
  WRITE(21,*) ' (°<  ERROR: Crystallographic vectors are not orthogonal:'
  WRITE(21,*) ' (°<                  [X,Y] =', orxy
  WRITE(21,*) ' (°<                  [X,Z] =', orxz
  WRITE(21,*) ' (°<                  [Y,Z] =', oryz
  WRITE(21,*) ' (°<         Program will EXIT.'
  WRITE(*,*) ' (°<  ERROR: Crystallographic vectors are not orthogonal.'
  WRITE(*,*) ' (°<         Program will EXIT.'
  counterror = counterror+1
  GOTO 1100
ENDIF
! Calculate base vectors length for this crystal orientation
ax = a0*DSQRT( REAL(orx(1))**2.d0+REAL(orx(2))**2.d0+REAL(orx(3))**2.d0 )/2.d0
ay = a0*DSQRT( REAL(ory(1))**2.d0+REAL(ory(2))**2.d0+REAL(ory(3))**2.d0 )/2.d0
az = a0*DSQRT( REAL(orz(1))**2.d0+REAL(orz(2))**2.d0+REAL(orz(3))**2.d0 )/2.d0
WRITE(21,*) '  Base vectors length (A):'
WRITE(21,'(a9,f12.6)') '    ax = ', ax
WRITE(21,'(a9,f12.6)') '    ay = ', ay
WRITE(21,'(a9,f12.6)') '    az = ', az
! Read atom spieces (only Al implemented yet)
READ(11,*,ERR=22) atomsp
IF(atomsp .NE. 'Al') GOTO 22
! Read atom spieces
WRITE(21,*) '  Atom spieces: ', atomsp
! Read deformation % 
READ(11,*,ERR=22) allon
allon = allon*0.01d0
WRITE(21,'(a16,f12.6,a2)') '   Deformation: ', 100.d0*allon, ' %'
! Read Poisson's ratio
READ(11,*,ERR=22) fish
WRITE(21,*) "  Poisson's ratio: ", fish
! Calculate base vectors of deformed crystal
ax = ax*(1.d0+allon)
ay = ay*(1.d0-fish*allon)
az = az*(1.d0-fish*allon)
! Read crystal dimension along each direction
READ(11,*,ERR=22) nx          ! Total number of layers in X direction
READ(11,*,ERR=22) ny          ! Total number of layers in Y direction
READ(11,*,ERR=22) nz          ! Total number of layers in Z direction
WRITE(21,'(a23,i3,a3,i3,a3,i3,a7)')     &
     &  '   Crystal dimensions: ', nx, ' x ', ny, ' x ', nz, ' layers'
GOTO 12
!
17 CONTINUE
! This is used to restrain the analysis to a certain {111} plane
! This is useful to save computational time when the crystal is too big,
! but it requires to know the coordinates of the interesting plane.
WRITE(21,*) ' '
WRITE(21,*) 'RESTRAINING ANALYSIS:'
! Does user want to force the writing in output file,
! no matter if the program finds the plane(s) interesting or not
! (0 = let the program decide if a plane is "interesting")
! (1 = forces the program to output displacements for    )
! (    selected planes, may they be "interesting" or not )
READ(11,*,ERR=22) forceoutput
IF(forceoutput==1) THEN
  WRITE(21,*) '  Output of displacement files [*.dat] will be forced for all selected atoms'
ELSE
  forceoutput = 0
ENDIF
! Check if GnuPlot is to be used
READ(11,*,ERR=22) gnuplot
IF(gnuplot==1) THEN
  WRITE(21,*) '  User declared that GnuPlot is available.'
ELSE
  gnuplot = 0
  WRITE(21,*) '  User declared that GnuPlot is not available'
  WRITE(21,*) ' (°<  WARNING: you specified not to use GnuPlot.'
  WRITE(21,*) ' (°<           Dislocation core position, speed and width may be inaccurate.'
  WRITE(22,*) ' (°<  WARNING: you specified not to use GnuPlot.'
  WRITE(22,*) ' (°<           Dislocation core position, speed and width may be inaccurate.'
  countwarning = countwarning+1
ENDIF
! Read value of the slope to investigate
! (1=positive slope; 2=negative; else=parse both)
READ(11,*,ERR=22) sslopeid
IF(sslopeid==1) THEN
  WRITE(21,*) '  Analyse only positive slope: slopeid=1'
ELSEIF(sslopeid==2) THEN
  WRITE(21,*) '  Analyse only negative slope: slopeid=2'
ELSE
  sslopeid = 0
  sidec = 0
  sidz = 0
  WRITE(21,*) '  No restriction - Analyse all planes'
  GOTO 12
ENDIF
! Value of idec to investigate (label of {111} plane)
READ(11,*,ERR=22) sidec
IF(sidec.NE.0) THEN
  WRITE(21,*) '  Analyse only plane number idec=', sidec
ELSE
  sidec = 0
  sidz = 0
  WRITE(21,*) '  Analyse all {111} planes with specified slope'
  GOTO 12
ENDIF
! Value of idz to investigate (label of Z-layer)
! (0=parse all Z-layers of the specified {111} plane)
READ(11,*,ERR=22) sidz
IF(sidz.NE.0) THEN
  WRITE(21,*) '  Analysing only Z-layer number idz=', sidz, ' in the specified {111} plane'
ELSE
  sidz = 0
  WRITE(21,*) '  Analysing all Z-layers in specified {111} plane'
ENDIF
! read glide plane type to analyze (will only be used if lattice==diamond)
READ(11,*,ERR=22) diamondgp
IF(TRIM(diamondgp).NE.'shuffle' .AND. TRIM(diamondgp).NE.'glide') THEN
  diamondgp = 'both'
ENDIF
! read which files are to be output
READ(11,'(3i1)',ERR=22) outedgef, outscrewf, outorthof
IF(outedgef==1) THEN
  WRITE(21,*) '  Displacements along the edge direction will be output'
ELSE
  outedgef = 0
  WRITE(21,*) '  Displacements along the edge direction will NOT be output'
ENDIF
IF(outscrewf==1) THEN
  WRITE(21,*) '  Displacements along the screw direction will be output'
ELSE
  outscrewf = 0
  WRITE(21,*) '  Displacements along the screw direction will NOT be output'
ENDIF
IF(outorthof==1) THEN
  WRITE(21,*) '  Displacements along the orthogonal direction will be output'
ELSE
  outorthof = 0
  WRITE(21,*) '  Displacements along the orthogonal direction will NOT be output'
ENDIF
READ(11,'(2i1)',ERR=22) outcoreposf, outcorewidthf
IF(outcoreposf==1) THEN
  WRITE(21,*) '  Core position in the {111} plane will be output'
ELSE
  outcoreposf = 0
  WRITE(21,*) '  Core position in the {111} plane will NOT be output'
ENDIF
READ(11,'(2i1)',ERR=22) outvelocityf
IF(outcoreposf==1) THEN
  WRITE(21,*) '  Dislocation velocity will be output'
ELSE
  outcoreposf = 0
  WRITE(21,*) '  Dislocation velocity will NOT be output'
ENDIF
GOTO 12
!
18 CONTINUE
recanal = 1
WRITE(21,*) ' '
WRITE(21,*) 'PERFORMING RECURSIVE ANALYSIS:'
reccount = 0
READ(11,'(a20)',ERR=22) recfileprefix
READ(11,'(a20)',ERR=22) recfilesuffix
! Read starting value of N for recursive analysis
READ(11,*,ERR=22) rnstart
rninc1 = rnstart
! Read starting value of N for recursive analysis
READ(11,*,ERR=22) rnend
IF(rnend>=100000 .OR. rnend==0) rnend = 99999
IF(rnstart<0 .OR. rnend<0) THEN
  WRITE(*,*) ' (°<  WARNING: starting and ending values for recursive analysis cannot'
  WRITE(*,*) ' (°<           be negative; they will be set to extrema values.'
  WRITE(21,*) ' (°<  WARNING: starting and ending values for recursive analysis cannot'
  WRITE(21,*) ' (°<           be negative; they will be set to extrema values..'
  rnstart = 0
  rnend = 99999
  countwarning = countwarning+1
ELSEIF(rnstart>=rnend) THEN
  WRITE(*,*) ' (°<  WARNING: starting value for recursive analysis cannot be greater'
  WRITE(*,*) ' (°<           than ending value; they will be set to extrema values.'
  WRITE(21,*) ' (°<  WARNING: starting value for recursive analysis cannot be greater'
  WRITE(21,*) ' (°<           than ending value; they will be set to extrema values.'
  rnstart = 0
  rnend = 99999
  countwarning = countwarning+1
ENDIF
!Read increment for N
READ(11,*,ERR=22) rninc
IF(rninc<=0) guessrninc=.TRUE.
WRITE(21,*) " All files named '", TRIM(recfileprefix)//'N'//    &
     &      TRIM(recfilesuffix)//"' will be searched,"
WRITE(21,'(a23,i6,a3,i6)') '   with N ranging from ', rnstart, ' to', rnend
WRITE(21,*) '  with an increment of ', rninc
recindex = rnstart
ALLOCATE(DPortho( INT(REAL(rnend-rnstart)/REAL(rninc))+5 ))
ALLOCATE(DPparaL( INT(REAL(rnend-rnstart)/REAL(rninc))+5 ))
ALLOCATE(DPparaR( INT(REAL(rnend-rnstart)/REAL(rninc))+5 ))
ALLOCATE(corewidthT( INT(REAL(rnend-rnstart)/REAL(rninc))+5 ))
DPortho(:) = 0.d0
DPparaL(:) = 0.d0
DPparaR(:) = 0.d0
corewidthT(:) = 0.d0
READ(11,*,ERR=22) dtime
WRITE(21,'(a30,f9.6,a4)') ' Time step between two files :', dtime, ' ps.'
GOTO 12
!
20 CONTINUE
IF(recanal==0 .OR. (recanal==1 .AND. recindex==0)) THEN
  CLOSE(11)
  !
  WRITE(21,*) '  Calculated edge and screw components are normalized'
  WRITE(21,*) '    to a perfect 60 dislocation components.'
  WRITE(21,*) ' '
ENDIF
!
IF(recanal==1) THEN
  IF(recindex<100000) THEN
    WRITE(rich5,'(i5)') recindex
    rich5 = ADJUSTL(rich5)
    xyzR = TRIM(recfileprefix)//TRIM(rich5)//TRIM(recfilesuffix)
  ELSE
    GOTO 600
  ENDIF
  IF(xyzR==xyzNR) THEN
    WRITE(21,*) "File '"//TRIM(xyzR)//"' is the same as unrelaxed crystal => skipping..."
    recindex = recindex+1
    GOTO 20
  ENDIF
  INQUIRE(FILE=xyzR,EXIST=exrecfile)
  IF(.NOT.exrecfile) THEN
    WRITE(21,*) "File '"//TRIM(xyzR)//"' does not exist => skipping..."
    recindex = recindex+1
    GOTO 20
  ELSE
    WRITE(21,*) ' '
    WRITE(21,*) '  >>> Found file ', xyzR
    IF(guessrninc .AND. rninc.NE.rnincmem) THEN
      rninc1 = rninc2
      rninc2 = recindex
      rnincmem = rninc2-rninc1
      rninc = rnincmem
    ENDIF
    reccount = reccount+1
  ENDIF
ENDIF
IF(rninc==0.d0) rninc=1
!
!
IF(recindex==1 .OR. recindex==rnstart) THEN
  WRITE(22,*) '  Read unrelaxed crystal atomic positions: ', xyzNR
  IF(recanal==0) THEN
    WRITE(22,*) '  Read relaxed crystal atomic positions: ', xyzR
  ELSE
    WRITE(22,*) '  Relaxed crystal atomic positions will be extracted from successive files'
  ENDIF
  WRITE(22,'(a16,f6.3,a2)') '  Deformation: ', allon*100, ' %'
ENDIF
!
IF(sslopeid.NE.1 .AND. sslopeid.NE.2) THEN
  sslopeid = 0
  sidec = 0
  sidz = 0
  IF(recindex==1 .OR. recindex==rnstart) THEN
    WRITE(22,*) '  No restriction - Analysing all planes in the crystal'
  ENDIF
  GOTO 25
ELSE
  IF(recindex==1 .OR. recindex==rnstart) THEN
    WRITE(22,*) ' '
    WRITE(22,*) 'RESTICTIONS ON PLANES TO BE PARSED :'
  ENDIF
  IF(sslopeid==1 .AND. (recindex==1 .OR. recindex==rnstart) ) THEN
    WRITE(22,*) '  Analyse only positive slope: slopeid = 1'
  ELSEIF(sslopeid==2 .AND. (recindex==1 .OR. recindex==rnstart) ) THEN
  WRITE(22,*) '  Analyse only negative slope: slopeid = 2'
  ENDIF
ENDIF
IF(sidec==0) GOTO 25
IF(recindex==1 .OR. recindex==rnstart) THEN
  IF(sidec==0) THEN
    WRITE(22,*) '  Analyse all {111} planes of specified slope'
  ELSE
    WRITE(22,*) '  Analyse only plane number idec = ', sidec
  ENDIF
ENDIF
IF(sidz.NE.0 .AND. (recindex==1 .OR. recindex==rnstart) ) THEN
  WRITE(22,*) '  Analyse only Z-layer  idz = ', sidz, ' in the specified {111} plane'
ELSEIF(recindex==1 .OR. recindex==rnstart) THEN
  WRITE(22,*) '  Analyse all Z-layers in specified {111} plane'
ENDIF
IF(forceoutput==1 .AND. (recindex==1 .OR. recindex==rnstart)) THEN
  WRITE(22,*) '  Output of displacement files [*.dat] will be forced for all selected atoms'
ENDIF
IF(recanal==1 .AND. (recindex==1 .OR. recindex==rnstart)) THEN
  WRITE(22,*) ' '
  WRITE(22,*) 'PERFORMING A RECURSIVE ANALYSIS'
ENDIF
GOTO 25
!
22 CONTINUE
   WRITE(21,*) " (°<  ERROR: A problem occured while attempting to read the"
   WRITE(21,*) " (°<         input file:  ", infile
   WRITE(21,*) ' >>>  Program will EXIT'
   WRITE(*,*) " (°<  ERROR: A problem occured while attempting to read the"
   WRITE(*,*) " (°<         input file: ", infile
   WRITE(*,*) ' >>>  Program will EXIT'
   counterror = counterror+1
   GOTO 1000
!
25 CONTINUE
! Initialize other variables
icore = 0
idec = 1         ! Initialise X-plane label
idz = 1          ! Initialize Z-layer label
intplane = 0     ! Initialize interesting planes label
delta = 0.01d0   ! 1/100 Angstrom
nbintpl = 0      ! Initializes number of interesting planes
idecorig = 200   ! Sets original shift to arbitrary value
natcmax = 0      ! Initializes maximum number of atoms in a layer
natcup = 0       ! Initializes maximum number of atoms over all layers
!
axs2 = 0.5d0*ax
bedge = a0*DSQRT(6.d0)/4.d0    ! Edge component of a perfect 60 dislocation
bscrew = a0/(2.d0*DSQRT(2.d0)) ! Screw component of a perfect 60 dislocation
d111 = a0/DSQRT(3.d0)          ! Distance between two (111) planes
htz = nz*az                    ! Thickness of the crystal (A)
pente = ax / (2.d0*ay)         ! Slope of {111} planes
pinv = 1.d0/pente              ! Inverse of the slope
co = ax / DSQRT(ax*ax + 4.d0*ay*ay)          !  Trigo
si = (2.d0*ay) / DSQRT(ax*ax + 4.d0*ay*ay)   !
!
ALLOCATE(coremem(20,nz))       ! Dislocation core position for idz=1,nz in
                               ! current {111} plane
coremem(:,:) = 0.d0
!
! Reading number of atoms from atomic positions files
currfile=xyzNR
OPEN(UNIT=10,FILE=xyzNR,ERR=33,STATUS='OLD')
currfile=xyzR
OPEN(UNIT=12,FILE=xyzR,ERR=33,STATUS='OLD')
GOTO 34
!
33 WRITE(21,*) " (°<  ERROR: position file '"//TRIM(currfile)//"'"
   WRITE(21,*) " (°<         is missing or has bad structure."
   WRITE(21,*) ' >>>  Program will EXIT.'
   WRITE(*,*) " (°<  ERROR: position file '"//TRIM(currfile)//"'"
   WRITE(*,*) " (°<         is missing or has bad structure."
   WRITE(*,*) ' >>>  Program will EXIT.'
   counterror = counterror+1
   GOTO 1000
!
34 CONTINUE
READ(10,*,ERR=33) particle, cfgtemp
IF(particle=='PARTICLE') THEN
  xyzNRtype = 'xyz'
  WRITE(21,*) " Non-relaxed position file is assumed to be xMD's output format [.xyz]:"
  WRITE(21,*) '   ', xyzNR
  REWIND(10)  ! Goes back to the beginning of file
  READ(10,*,ERR=33) particle1, natom1
ELSEIF(TRIM(particle)=='Number') THEN
  xyzNRtype = 'cfg'
  WRITE(21,*) " Non-relaxed position file is assumed to be Ju Li's format [.cfg]:"
  WRITE(21,*) '   ', xyzNR
  REWIND(10)
  READ(10,'(a21,i)',ERR=33) particlecfg, natom1
ELSE
  GOTO 33
ENDIF
!
READ(12,*,ERR=33)  particle, cfgtemp
IF(particle=='PARTICLE') THEN
  xyzRtype = 'xyz'
  WRITE(21,*) " Relaxed position file is assumed to be xMD's output format [.xyz]:"
  WRITE(21,*) '   ', xyzR
  REWIND(12)
  READ(12,*,ERR=33) particle2, natom2
ELSEIF(TRIM(particle)=='Number') THEN
  xyzRtype = 'cfg'
  WRITE(21,*) " Relaxed position file is assumed to be Ju Li's format [.cfg]:"
  WRITE(21,*) '   ', xyzR
  REWIND(12)
  READ(12,'(a21,i)',ERR=33) particlecfg, natom2
ELSE
  GOTO 33
ENDIF
!
WRITE(21,'(a50,i6,a3,i6)') " Number of atoms in unrelaxed/relaxed crystals = ", natom1,  &
     &                     ' / ', natom2
! Check if number of atoms is correct
IF(natom1 .NE. natom2) THEN
  WRITE(*,*) ' '
  WRITE(*,*) ' (°<  ERROR: The number of atoms in the two systems do not match.'
  WRITE(*,*) " (°<         Please check that positions files '"//TRIM(xyzNR)//"' and"
  WRITE(*,*) " (°<         '"//TRIM(xyzR)//"' refer to the same crystal."
  WRITE(*,*) ' >>>  PROGRAM EXIT.'
  WRITE(21,*) ' '
  WRITE(21,*) ' (°<  ERROR: The number of atoms in the two systems do not match.'
  WRITE(21,*) " (°<         Please check that positions files '"//TRIM(xyzNR)//"' and"
  WRITE(21,*) " (°<         '"//TRIM(xyzR)//"' refer to the same crystal."
  WRITE(21,*) ' >>>  PROGRAM EXIT.'
  counterror = counterror+1
  GOTO 1000
ENDIF
!
natom = natom1
IF(recindex==1 .OR. recindex==rnstart) THEN
  WRITE(22,*) " Number of atoms in the crystal = ", natom
ENDIF
!
ALLOCATE(coordNR(3,natom), coordR(3,natom))
!
! Reading atomic positions from positions file
! If file format is Ju Li's CFG then use transformation matrix
currfile = xyzNR
IF(xyzNRtype=='xyz') THEN
  DO n = 1, natom
    READ(10,*,END=35,ERR=33) itruc, coordNR(1,n), coordNR(2,n), coordNR(3,n)
  ENDDO
ELSE  ! ie, xyzNRtype == 'cfg'
  READ(10,'(a3,f,a)',END=33,ERR=33) cfgAtxt, cfgA, cfgAtxt
  DO m=1,9
    READ(10,'(a9,f)',END=35,ERR=33) mattxt, mvalue
    IF(mattxt=='H0(1,1) =') THEN
      H0(1,1) = mvalue
    ELSEIF(mattxt=='H0(1,2) =') THEN
      H0(1,2) = mvalue
    ELSEIF(mattxt=='H0(1,3) =') THEN
      H0(1,3) = mvalue
    ELSEIF(mattxt=='H0(2,1) =') THEN
      H0(2,1) = mvalue
    ELSEIF(mattxt=='H0(2,2) =') THEN
      H0(2,2) = mvalue
    ELSEIF(mattxt=='H0(2,3) =') THEN
      H0(2,3) = mvalue
    ELSEIF(mattxt=='H0(3,1) =') THEN
      H0(3,1) = mvalue
    ELSEIF(mattxt=='H0(3,2) =') THEN
      H0(3,2) = mvalue
    ELSEIF(mattxt=='H0(3,3) =') THEN
      H0(3,3) = mvalue
    ELSE
      GOTO 33
    ENDIF
  ENDDO
  IF(DABS(H0(1,1))<=1.d0 .OR. DABS(H0(2,2))<=1.d0 .OR. DABS(H0(3,3))<=1.d0) GOTO 37
  READ(10,*,ERR=33) cfgtemp
  READ(10,*,ERR=33) cfgtemp
  READ(10,*,ERR=33) atommass
  READ(10,*,ERR=33) atomsp
  DO n=1,natom
    READ(10,*,END=35,ERR=33) cfgx, cfgy, cfgz
    coordNR(1,n) = cfgx*H0(1,1)  !*cfgA
    coordNR(2,n) = cfgy*H0(2,2)  !*cfgA
    coordNR(3,n) = cfgz*H0(3,3)  !*cfgA
  ENDDO
ENDIF
CLOSE(10)
WRITE(21,*) ' >>>  Done reading atomic positions from file ', TRIM(xyzNR)
!
! Same with relaxed file :
currfile = xyzR
IF(xyzRtype=='xyz') THEN
  DO n = 1, natom
    READ(12,*,END=35) itruc, coordR(1,n), coordR(2,n), coordR(3,n)
  ENDDO
ELSE  ! ie, xyzRtype == 'cfg'
  READ(12,'(a3,f,a)',END=33,ERR=33) cfgAtxt, cfgA, cfgAtxt
  DO m=1,9
    READ(12,'(a9,f)',END=35,ERR=33) mattxt, mvalue
    IF(mattxt=='H0(1,1) =') THEN
      H0(1,1) = mvalue
    ELSEIF(mattxt=='H0(1,2) =') THEN
      H0(1,2) = mvalue
    ELSEIF(mattxt=='H0(1,3) =') THEN
      H0(1,3) = mvalue
    ELSEIF(mattxt=='H0(2,1) =') THEN
      H0(2,1) = mvalue
    ELSEIF(mattxt=='H0(2,2) =') THEN
      H0(2,2) = mvalue
    ELSEIF(mattxt=='H0(2,3) =') THEN
      H0(2,3) = mvalue
    ELSEIF(mattxt=='H0(3,1) =') THEN
      H0(3,1) = mvalue
    ELSEIF(mattxt=='H0(3,2) =') THEN
      H0(3,2) = mvalue
    ELSEIF(mattxt=='H0(3,3) =') THEN
      H0(3,3) = mvalue
    ELSE
      GOTO 33
    ENDIF
  ENDDO
  IF(DABS(H0(1,1))<=1.d0 .OR. DABS(H0(2,2))<=1.d0 .OR. DABS(H0(3,3))<=1.d0) GOTO 37
  READ(12,*,ERR=33) cfgtemp
  READ(12,*,ERR=33) cfgtemp
  READ(12,*,ERR=33) atommass
  READ(12,*,ERR=33) atomsp
  DO n=1,natom
    READ(12,*,END=35,ERR=33) cfgx, cfgy, cfgz
    coordR(1,n) = cfgx*H0(1,1)  !*cfgA
    coordR(2,n) = cfgy*H0(2,2)  !*cfgA
    coordR(3,n) = cfgz*H0(3,3)  !*cfgA
  ENDDO
ENDIF
CLOSE(12)
WRITE(21,*) ' >>>  Done reading atomic positions from file ', TRIM(xyzR)
!
GOTO 36
35 WRITE(21,*) " (°<  ERROR: The number of lines in coordinates file '"//TRIM(currfile)//"'"
   WRITE(21,*) " (°<         does not match the number of particles. Please check this file."
   WRITE(21,*) " >>>  Program EXIT."
   WRITE(*,*) " (°<  ERROR: The number of lines in coordinates file '"//TRIM(currfile)//"'"
   WRITE(*,*) " (°<         does not match the number of particles. Please check this file."
   WRITE(*,*) " >>>  Program will EXIT."
   counterror = counterror+1
   GOTO 1000
!
37 WRITE(21,*) ' (°<  ERROR: One or more box dimension is zero.'
   WRITE(21,*) ' >>>  Program will EXIT.'
   WRITE(*,*) ' (°<  ERROR: One or more box dimension is zero.'
   WRITE(*,*) ' >>>  Program will EXIT.'
   counterror = counterror+1
   GOTO 1000

36 CONTINUE
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Change basis set: to be implemented here !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! If the basis set is not ([0-11], [100], [011]) then
! rotate the crystal 
!  
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Output atomic positions for the Z-layer if specified
IF(sidz.NE.0 .AND. forceoutput==1) THEN
  zmin = idz*az
  zmax = zmin+az/2
  DO n=1,natom
    IF(coordR(3,n)>zmin-0.1d0*az .AND. coordR(3,n)<zmax+0.1d0*az) THEN
      nzcfgcount = nzcfgcount+1
    ENDIF
  ENDDO
  IF(recanal==0) THEN
    IF(idz<1000) THEN
      WRITE(ich3,'(i3)') idz
      ich3 = ADJUSTL(ich3)
      partcfgout = 'nz_'//TRIM(ich3)//'.cfg'
    ELSE
      GOTO 600
    ENDIF
  ELSE
    WRITE(*,*) ' '
    WRITE(*,*) ' >>>  RECURSIVE ANALYSIS, PART ', recindex
    IF(idz<1000 .AND. recindex<100000) THEN
      WRITE(ich3,'(i3)') idz
      WRITE(rich5,'(i5)') recindex
      ich3 = ADJUSTL(ich3)
      rich5 = ADJUSTL(rich5)
      partcfgout = 'nz_'//TRIM(ich3)//'_t'//TRIM(rich5)//'.cfg'
    ELSE
      GOTO 600
    ENDIF !idz
  ENDIF  ! recanal
  currfile = partcfgout
  OPEN(UNIT=31,FILE=partcfgout,FORM='FORMATTED',STATUS='UNKNOWN',ERR=35)
  WRITE(31,'(a21,i6)') 'Number of particles =', nzcfgcount
  WRITE(31,'(a9)') 'A = 1.000'
  WRITE(31,'(a9,f16.9)') 'H0(1,1) =', H0(1,1)
  WRITE(31,'(a18)') 'H0(1,2) =  0.00000'
  WRITE(31,'(a18)') 'H0(1,3) =  0.00000'
  WRITE(31,'(a18)') 'H0(2,1) =  0.00000'
  WRITE(31,'(a9,f16.9)') 'H0(2,2) =', H0(2,2)
  WRITE(31,'(a18)') 'H0(2,3) =  0.00000'
  WRITE(31,'(a18)') 'H0(3,1) =  0.00000'
  WRITE(31,'(a18)') 'H0(3,2) =  0.00000'
  WRITE(31,'(a9,f16.9)') 'H0(3,3) =', 1.2d0*az
  WRITE(31,'(a13)') '.NO_VELOCITY.'
  WRITE(31,'(a15)') 'entry_count = 3'
  WRITE(31,'(a9)') '27.000000'
  WRITE(31,'(a2)') atomsp
  DO n=1,natom
    IF(coordR(3,n)>zmin-0.1d0*az .AND. coordR(3,n)<zmax+0.1d0*az) THEN
      WRITE(31,'(3(f8.5,2X))') coordR(1,n)/H0(1,1), coordR(2,n)/H0(2,2),  &
           &      (coordR(3,n)-zmin)/az
    ENDIF
  ENDDO
  CLOSE(31)
  WRITE(*,*) ' >>>  WROTE FILE '//TRIM(partcfgout)
ENDIF
!
!
!========================================
! CHANGING BASE VECTORS
!========================================
!
! ## Unrelaxed crystal
! ####################
! We have to check that all atoms are within the box,
! ie. that z is between (zmin-az/4) and (zmin+nz*az-az/4)
!
! Determining minimum z coordinate
zmin = coordNR(3,1)
DO i = 2, natom
  IF(coordNR(3,i) < zmin) zmin = coordNR(3,i)
ENDDO
!
! Calculating zmax from zmin and box thickness
zmax = zmin + htz - 0.25d0*az
zmin0 = zmin
zmax0 = zmax
!
! Replacing out-of-box atoms into the box by translating them
! by one base vector htz
DO i=1,natom
  IF(coordNR(3,i) >= zmax) coordNR(3,i) = coordNR(3,i)-htz
ENDDO
!
! Set the origin to the middle of X axis and on the superior surface
! (=the surface with the steps)
xminNR = coordNR(1,1)
xmaxNR = coordNR(1,1)
!
DO i=1,natom
  IF (coordNR(1,i) < xminNR) xminNR = coordNR(1,i)
  IF (coordNR(1,i) > xmaxNR) xmaxNR = coordNR(1,i)
ENDDO
!
yminNR = coordNR(2,1)
ymaxNR = coordNR(2,1)
!
DO i=1,natom
  IF(coordNR(2,i) < yminNR) yminNR = coordNR(2,i)
  IF(coordNR(2,i) > ymaxNR) ymaxNR = coordNR(2,i)
ENDDO
WRITE(21,'(a23,f9.4,a3,f9.4,a3,f9.4)') '  BOX DIMENSIONS (A) : ',   &
     & (xmaxNR-xminNR), ' x ', (ymaxNR-yminNR), ' x ', zmax-zmin
!
x0 = 0.5d0*(xmaxNR+xminNR)
!
! New coordinates of origin : (x0,ymaxNR,0)
! atom mm : closest to (x0, ymaxNR)
! Calculating distance of each atom relative to this new origin
ALLOCATE(dist(natom))
DO n=1,natom
  dist(n) = (coordNR(1,n)-x0)**2.d0 + (coordNR(2,n)-ymaxNR)**2.d0
ENDDO
!
! Determining mm = index of the nearest atom from the origin
mm = 1
distmin = dist(1)
DO i=1,natom
  IF(dist(i) < distmin) THEN
    mm = i
    distmin = dist(i)
  ENDIF
ENDDO
DEALLOCATE(dist)
!
! Set X and Y coordinates for atom number mm
WRITE(21,*) ' '
WRITE(21,*) ' Express all atoms coordinates in the new base :'
tr1 = coordNR(1,mm)
tr2 = coordNR(2,mm)
WRITE(21,*) '      Translate X by ', tr1
WRITE(21,*) '      Translate Y by ', tr2
!
! Normalizing all coordinates of unrelaxed crystal
! to this new base
DO n=1,natom
  coordNR(1,n) = coordNR(1,n) - tr1
  coordNR(2,n) = coordNR(2,n) - tr2   !replace -ymaxNR
ENDDO
!
! ## The same with relaxed crystal
! ################################
!
! Set the origin to the middle of X axis and on the superior surface
! (=the surface with the steps)
!
zminR = coordR(3,1)
!
! Replace out-of-box atoms in the box
DO i=2,natom
  IF (coordR(3,i) < zminR) THEN
    zminR = coordR(3,i)
    outofbox = outofbox+1
  ENDIF
ENDDO
IF(outofbox.NE.0 .AND. outofboxNR==.FALSE.) THEN
  WRITE(21,'(a15,i6,a32)') ' (°<  WARNING: ', outofbox, ' out-of-box atoms were replaced'
  WRITE(21,*) ' (°<          in the unrelaxed crystal  '//TRIM(xyzNR)
  countwarning = countwarning+1
  outofboxNR = .TRUE.
ENDIF
outofbox = 0
zmaxR = zminR + htz - 0.25d0*az
!
DO i=1,natom
  IF (coordR(3,i) >= zmaxR) THEN
    coordR(3,i) = coordR(3,i)-htz
    outofbox = outofbox+1
  ENDIF
ENDDO
!
IF(outofbox.NE.0) THEN
  WRITE(21,'(a15,i6,a32)') ' (°<  WARNING:', outofbox, ' out-of-box atoms were replaced'
  WRITE(21,*) ' (°<          in the relaxed crystal  '//TRIM(xyzR)
  countwarning = countwarning+1
ENDIF
outofbox = 0
!
DO n=1,natom
  coordR(1,n) = coordR(1,n) - tr1
  coordR(2,n) = coordR(2,n) - tr2
ENDDO
!
WRITE(21,*) ' '
WRITE(21,*) ' >>>  All atoms coordinates were successfully transformed into new base'
WRITE(21,*) ' '
WRITE(21,*) ' '
WRITE(21,*) ' >>>  Start parsing {111} planes...'
WRITE(21,*) ' '
WRITE(*,*) ' '
WRITE(*,*) ' >>>  Start parsing {111} planes, it may take several minutes...'
IF(recanal==1) THEN
  WRITE(*,*) ' >>>  Recursive analysis, part ', recindex
ENDIF
!
!=============================================
! PARSING ALL {111} PLANES
!=============================================
!
! Short informations on the process progression will be output in
! the file "dechar_111.nfo"
OPEN(UNIT=25,FILE='dechar_111.nfo',FORM='FORMATTED',STATUS='UNKNOWN')
!
pllbl = 1  ! Labels planes
idecmax = NINT(2.d0*x0/ax)
!
! Gargantuous loop on slopeid = sign of the slope of the {111} planes parsed
40 CONTINUE
! 
! Check if user asked to restrain analysis to a certain slope :
IF(sslopeid.NE.0) THEN
  slopeid = sslopeid
ELSE
  slopeid = slopeid+1
ENDIF
!
! If we parse only planes from xmin to xmax, then only planes that reach
! the surface will be parsed ; in order to take all the planes into account,
! we have to add the crystal's height Y to the loop on idec
addyl = 0  ! No planes added on the left for slopeid=1
addyr = 1  ! This will add planes on the right for slopeid=1
! If slope is negative, then rotate the crystal by 180° about Y
IF(slopeid==2) THEN
  DO n=1,natom
    coordR(1,n) = -coordR(1,n)
    coordNR(1,n) = -coordNR(1,n)
    coordR(3,n) = -coordR(3,n)+REAL(nz)*az
    coordNR(3,n) = -coordNR(3,n)+REAL(nz)*az
  ENDDO
ENDIF
!
WRITE(21,*) ' '
WRITE(21,*) '=================================================='
WRITE(21,*) ' PARSING {111} PLANES, PART ', slopeid
WRITE(21,*) '=================================================='
IF(recindex==1) THEN
  WRITE(22,*) ' '
  WRITE(22,'(a28,i2)') ' PARSING {111} PLANES, PART ', slopeid
ENDIF
IF(slopeid==1) THEN
  WRITE(21,*) '   Slope is positive.'
  IF(recindex==1) THEN
    WRITE(22,*) '===========  (Slope is positive)  ===================='
  ENDIF
ELSE
  WRITE(21,*) '   Slope is negative.'
  IF(recindex==1) THEN
    WRITE(22,*) '===========  (Slope is negative)  ===================='
  ENDIF
ENDIF
WRITE(21,*) ' '
WRITE(25,*) 'PARSING {111} PLANES, PART ', slopeid
!
! Enormous loop on idec = parsing all {111} planes with defined slope
idec = -NINT(x0/ax)-addyl*(ny/2)-1
!
50 CONTINUE
!
! Check if user restrained analysis to a specific {111} plane
IF(sidec.NE.0) THEN
  idec = sidec
ELSE
  idec = idec+1
ENDIF
idecorig = idec
isintplane = 0
intplane = intplane+1
!
WRITE(21,*) '     ################################################'
WRITE(21,*) '   ####                                            ####'
WRITE(21,'(a41,i3,a15)') '#######         {111}  PLANE  NUMBER  :  ', idec, '        #######'
WRITE(21,*) '   ####                                            ####'
WRITE(21,*) '     ################################################'
!
! ## OPERATIONS ON UNRELAXED CRYSTAL ########
! ###########################################
!
! ~ Cutting off two {111} planes
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! The first plane is distant of idec*ax from the origin.
! The second plane is adjascent to the first one.
! A distance equal to ax/4 is added on each side to take
! eventual out-of-plane minifluctuations into account.
xdec(1) = idec*ax-ax/4.d0
xdec(2) = (idec+1)*ax+ax/4.d0
!
DO k=1,2
  p1(k) = xdec(k) - axs2
  p2(k) = xdec(k) + axs2
ENDDO
!
! Counting atoms in {111} planes
natp(:) = 0
DO k=1, 2  ! Loop on planes 1 and 2
  natp(k) = 0
  DO n=1,natom  ! Loop on all atoms
    prox = coordNR(1,n) - pente*coordNR(2,n)
    IF ((prox > p1(k)) .AND. (prox < p2(k))) THEN
      natp(k) = natp(k)+1
    ENDIF
  ENDDO
  WRITE(21,*) '      {111} plane number ', k, ' counts ', natp(k), ' atoms.'
ENDDO
natpmax = MAX(natp(1),natp(2))
WRITE(21,*) '=> Max nb of atoms in planes : ', natpmax
WRITE(21,*) '******************************'
WRITE(21,*) ' '
!
! Determining px = positions of atoms in {111} planes
ALLOCATE(px(2,3,natpmax))
!ALLOCATE(atomN(2,natpmax))
!
DO k=1, 2  ! Loop on planes 1 and 2
  ii = 0  ! Indexing atoms in the planes
  DO n=1,natom  ! Loop on all atoms
    prox = coordNR(1,n) - pente*coordNR(2,n)
    IF ((prox > p1(k)) .AND. (prox < p2(k))) THEN
      !atomN(k,ii) = n
      ii = ii+1
      px(k,1,ii) = coordNR(1,n)  ! Coordinates of the
      px(k,2,ii) = coordNR(2,n)  ! ii-th atom of the
      px(k,3,ii) = coordNR(3,n)  ! k-th plane
    ENDIF
  ENDDO  !n
ENDDO  !k
!
!
! ~~ Cutting off an az thick layer
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! Huge loop on idz = layers orthogonal to Z direction
idz = 0
60 CONTINUE
!
IF(intplane>=20) THEN
  WRITE(21,*) ' '
  WRITE(21,*) ' (°<  ERROR: a problem was detected'
  WRITE(21,*) ' (°<         Your crystal seems to contain a lot of'
  WRITE(21,*) ' (°<         activated glide planes. This program is designed'
  WRITE(21,*) ' (°<         to analyze crystals which present only a few of such'
  WRITE(21,*) ' (°<         planes and will terminate.'
  WRITE(25,*) ' '
  WRITE(25,*) ' (°<  ERROR: a problem was detected'
  WRITE(25,*) ' (°<         Your crystal seems to contain a lot of'
  WRITE(25,*) ' (°<         activated glide planes. This program is designed'
  WRITE(25,*) ' (°<         to analyze crystals which present only a few of such'
  WRITE(25,*) ' (°<         planes and will terminate.'
  WRITE(22,*) ' '
  WRITE(22,*) ' (°<  ERROR: a problem was detected'
  WRITE(22,*) ' (°<         Your crystal seems to contain a lot of'
  WRITE(22,*) ' (°<         activated glide planes. This program is designed'
  WRITE(22,*) ' (°<         to analyze crystals which present only a few of such'
  WRITE(22,*) ' (°<         planes and will terminate.'
  WRITE(*,*) ' (°<  ERROR: a problem was detected'
  WRITE(*,*) ' (°<         Your crystal seems to contain a lot of'
  WRITE(*,*) ' (°<         activated glide planes. This program is designed'
  WRITE(*,*) ' (°<         to analyze crystals which present only a few of such'
  WRITE(*,*) ' (°<         planes and will terminate.'
  counterror = counterror+1
  GOTO 500
ENDIF
!
! Check if user restrained analysis to a specific Z-layer
IF(sidz.NE.0) THEN
  idz = sidz
ELSE
  idz = idz+1
ENDIF
!
WRITE(21,*) ' '
WRITE(21,'(a17,i1,a13,i3,a16,i3)') '        SLOPEID= ', slopeid, '   ;   IDEC= ', idec,        &
     &                             '   ;   Z LAYER= ', idz
WRITE(21,*) '                -             ---                ---'
WRITE(21,*) ' '
!
! Reinitializing zmin and zmax
zmin = zmin0
zmax = zmax0
!
! Calculating zmin and zmax for layer #idz
zmin = zmin - 0.25d0*az
xidz = idz - 1
zmin = zmin + xidz*az
zmax = zmin + az
!
! Counting atoms in the layer
natc(:) = 0
DO k=1, 2  ! Loop on planes 1 and 2
  natc(k) = 0   ! 
  DO i=1,natp(k)
    IF ((px(k,3,i) >= zmin) .AND. (px(k,3,i) < zmax)) &
        &  natc(k)=natc(k)+1
  ENDDO
  WRITE(21,'(a13,i2,a3,i3,a7)') '     Layer # ', k, ' : ', natc(k), ' atoms.'
ENDDO
natcmax = MAX (natc(1), natc(2))
!
! Calculate maximum number of atoms over all the planes
IF(natcmax > natcup) THEN
  natcup = natcmax
ENDIF
WRITE(21,'(a47,i3)') '     => Maximum number of atoms in a Z-layer : ', natcmax
!
! Determining cx = coordinates of atoms in the layer
ALLOCATE(cx(2,3,natcmax))
cx(:,:,:)=0.d0
!
DO k=1,2  ! Loop on planes 1 and 2
  jj = 0  ! Indexing atoms in the layer
  DO i=1,natp(k)
    IF ((px(k,3,i) >= zmin) .AND. (px(k,3,i) < zmax)) THEN
      jj = jj+1
      DO l=1,3
        cx (k,l,jj) = px (k,l,i)
      ENDDO
    ENDIF
  ENDDO
ENDDO ! k
!
!
! ~~ Sorting atoms by decreasing Y-coordinate
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! NB: yc are the Y-coordinates of the cx
!
ALLOCATE(yc(2, natcmax), ir(2,natcmax), ir1(2,natcmax), &
       & yd(2,natcmax), cry(2,3,natcmax), crx0(2,3,natcmax))
yc(:,:) = 0.d0
ir(:,:) = 0.d0
ir1(:,:) = 0.d0
yd(:,:) = 0.d0
cry(:,:,:) = 0.d0
crx0(:,:,:) = 0.d0
!
DO k=1, 2
  DO j=1,natc(k)
    yc(k,j)=cx(k,2,j)
    ir(k,j)=j
  ENDDO
!
  DO j=1,natc(k)
    yd(k,j) = yc(k,j)
    i2 = j
    DO i=j,natc(k)
      IF(yc(k,i) > yd(k,j)) THEN
        yd(k,j) = yc(k,i)
        i2 = i
      ENDIF
    ENDDO
!
    ! We shift the yc(k,i) for i from j to i2
    IF(i2 > j) THEN
      DO i=i2,j,-1
        yc(k,i) = yc(k,i-1)
      ENDDO
    ENDIF

    ir1 (k,j) = ir (k,i2)
!
    IF(i2 > j) THEN
      DO i=i2,j,-1
        ir (k,i) = ir (k,i-1)
      ENDDO ! i
    ENDIF
  ENDDO  !j
!
!  NB: cry are the sorted cx by increasing Y
  DO j=1,natc(k)
    DO i=1, 3
      cry(k,i,j) =  cx(k,i,natc(k)+1-j)
    ENDDO
  ENDDO
ENDDO  ! End loop on planes k
!
WRITE(21,*) ' >>>  Sorted atoms in current {111} planes.'
!
! Identifying atoms in the layer
ALLOCATE(iatom(2,natcmax))
iatom(:,:) = 0
!
DO k=1,2
  DO j=1,natc(k)
    DO n=1,natom
      IF(cry(k,1,j) .NE. coordNR(1,n) ) CYCLE
      IF(cry(k,2,j) .NE. coordNR(2,n) ) CYCLE
      IF(cry(k,3,j) .NE. coordNR(3,n) ) CYCLE
      iatom(k,j) = n
    ENDDO  !n
  ENDDO
ENDDO
!
! ~~ Calculating relative displacements (unrelaxed crystal)
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
natcmin = MIN(natc(1),natc(2))
!
ALLOCATE(ddep(3,natcmin), ddep2(3,natcmin), d(3,natcmin))
ddep(:,:) = 0.d0
ddep2(:,:) = 0.d0
d(:,:) = 0.d0
IF((idz==1) .OR. (sidz.NE.0)) THEN
  ALLOCATE(dscrew(nz,natpmax), dedge(nz,natpmax), dortho(nz,natpmax))
  dscrew(:,:) = 0.d0
  dedge(:,:) = 0.d0
  dortho(:,:) = 0.d0  
ENDIF
!
DO i=1,natcmin
    DO j=1,3
      ddep(j,i) = cry(2,j,i)-cry(1,j,i)
    ENDDO
ENDDO
WRITE(21,*) ' >>> Calculated the relative positions in unrelaxed crystal.'
!
!
! ## SEARCHING THE SAME ATOMS IN THE RELAXED CRYSTAL
! ####################################################
!
DO k=1,2
  DO m=1,natcmin
    DO n=1,natom
      ! We check if unrelaxed and relaxed atom labels match :
      IF(n == iatom(k,m)) THEN
        DO i=1,3
          cry(k,i,m) = coordR(i,n)
        ENDDO
      ENDIF
    ENDDO
  ENDDO
ENDDO
!
WRITE(21,*) ' >>>  Found corresponding atoms in relaxed crystal.'
!
! ~~ Calculating relative displacements (relaxed crystal)
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
DO i=1,natcmin
  DO j=1,3
    ddep2(j,i) = cry(2,j,i) - cry(1,j,i)
    ! Difference in relative displacement
    d(j,i) = ddep2(j,i) - ddep(j,i)
  ENDDO
ENDDO
!
! ~~ Declare and initialize values of definitive displacements
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
IF(donefinal==0) THEN
    ALLOCATE(dedgefinal(20,nz,natpmax))
    ALLOCATE(dscrewfinal(20,nz,natpmax))
    ALLOCATE(dorthofinal(20,nz,natpmax))
    ! 3 parameters : label of the interesting plane, label of the Z-layer
    ! within this plane, and label of the atom (#1 is at the surface).
    ! With this we limit the maximum number of interesting planes to 20,
    ! and parse all atoms in all the Z-layers within each plane.
    ! Maximum number of atoms is fixed to max.number of atoms in a layer
    ! times  max.number of interesting layers.
    dedgefinal(:,:,:) = 0.d0
    dscrewfinal(:,:,:) = 0.d0
    dorthofinal(:,:,:) = 0.d0
    donefinal = 1
    pllbl=pllbl+1
ENDIF
!
!=====================================================
! IDENTIFYING SCREW AND EDGE COMPONENTS
! OF THE DISPLACEMENTS AND OUTPUT RESULTS
!=====================================================
! If dedge = 2/3 => 90 partial dislocation
! If dedge = 1/3 => 30 partial dislocation
! If dedge = 1   => 60 perfect dislocation
! If dedge = 0   => No dislocation
!
WRITE(21,'(a63,a27)') 'Atom label            Edge component            Screw component' , &
        &             '           Normal component'
WRITE(21,'(a63,a27)') '----------            --------------            ---------------' , &
        &             '           ----------------'
icore = 0
coredone = 0
DO i=1,natcmin
   dscrew(idz,i) = d(3,i) /bscrew         ! Screw component of the dislocation
   dedge(idz,i) = (d(1,i)*DSIN(35.d0) + (d(2,i)*DCOS(35.d0))) /bedge ! Edge comp.
   dortho(idz,i) = (d(1,i)*DCOS(35.d0) - d(2,i)*DSIN(35.d0)) /d111
   IF(DABS(dscrew(idz,i)) > REAL(nz-1)) THEN ! This means an atom moved to the
     dscrew(idz,i) = 0.d0                    ! neighbouring slab and was replaced
   ELSEIF(DABS(dscrew(idz,i)) > 2.d0) THEN
     warning = 1                                !
     dscrew(idz,i) = 0.d0                       ! This prevents artificially huge
   ENDIF                                        !
   IF(DABS(dedge(idz,i)) > REAL(nz-1)) THEN     ! displacements and will warn user
     warning = 1                                !
     dedge(idz,i)  = 0.d0                       !
   ELSEIF(DABS(dedge(idz,i)) > REAL(nz-1)) THEN !
     dedge(idz,i)  = 0.d0                       !
   ENDIF
   IF(DABS(dortho(idz,i)) > 5.d0) THEN
     dortho(idz,i) = 0.d0
   ELSEIF(DABS(dortho(idz,i)) > 1.5d0) THEN
     warning2 = 1
     dortho(idz,i) = 0.d0
   ENDIF
   WRITE(21,'(i2,a10,d24.17,a3,d24.17,a3,d24.17)') i, '          ', dedge(idz,i),   &
           &                    '   ', dscrew(idz,i), '   ', dortho(idz,i)
   ! Find dislocation core position from the surface. The core is
   ! is here arbitrarily defined where the displacement equals
   ! 0.2 times the Burgers vector of a 60 perfect dislocation.
   ! (might not be a real dislocation core, since we don't know
   ! yet if we have a dislocation in this plane)
   IF(DABS(dedge(idz,i))<0.2d0) THEN
     coredone = 1
   ENDIF
   IF(coredone==0) THEN
     icore = icore+1
   ENDIF
   dedgefinal(intplane,idz,i) = dedge(idz,i)
   dscrewfinal(intplane,idz,i) = dscrew(idz,i)
   dorthofinal(intplane,idz,i) = dortho(idz,i)
   OPEN(100,FILE="core.list")
   IF(dedgefinal(intplane,idz,i)>0.2d0 .AND. dedgefinal(intplane,idz,i)<0.4d0) THEN
     WRITE(100,*) i
   ENDIF
ENDDO  ! End loop on i
WRITE(21,*) '---------------------------------------------------------------'
!
dis1 = DABS(dedge(idz,1))
dis2 = DABS(dedge(idz,2))
dis5 = DABS(dedge(idz,5))
dis11 = DABS(dedge(idz,natcmin-1))
dis12 = DABS(dedge(idz,natcmin-2))
dis15 = DABS(dedge(idz,natcmin-5))
!
! The dislocation is considered nucleated when its
! edge displacements become greater than 0.3*bedge(60).
IF(  recanal==1  .AND.  donedisfront==0  .AND. dis1>=0.4d0   &
   & .AND. dis2>=0.3d0 .AND. DABS(dedge(idz,3))>=0.2d0     ) THEN
  disfront = idz
  nucleationtime = recindex*dtime
  WRITE(*,*) '          >>> DISLOCATION NUCLEATION <<<'
  WRITE(25,*) '          >>> DISLOCATION NUCLEATION <<<'
  WRITE(21,*) '          >>> DISLOCATION NUCLEATION <<<'
  WRITE(21,*) ' >>>  Dislocation front detected in idz = ', disfront
  WRITE(21,*) ' >>>  Dislocation nucleation time was estimated at', nucleationtime, ' ps'
  isintplane = 1
  donedisfront = 1
ENDIF
!
! Guess dislocation identity
IF(   (dis1>0.1d0  .AND. dis2>0.1d0  .AND. dis5>0.1d0)    .OR.      &
  &   (dis11>0.1d0 .AND. dis12>0.1d0 .AND. dis15>0.1d0)             &
  &                .AND. icore>2     .AND. natcmax>12               ) THEN 
  WRITE(21,*) ' >>>  Guessing dislocation identity :'
  ! If displacement does not reach the 5th atom under
  ! the surface, it is not considered as a dislocation
  IF(dis11>0.1d0 .AND. dis12>0.1d0 .AND. dis15>0.1d0  .AND.         &
    &dis1<0.1d0  .AND. dis2<0.1d0  .AND. dis5<0.1d0                 ) THEN
    dis1 = dis11
    dis2 = dis12
    dis5 = dis15
  ENDIF
  IF(dis1<0.2) THEN
    WRITE(21,*) '    No dislocation'
    dis=0.d0
  ELSEIF((dis1>0.2) .AND. (dis2>0.59) .AND. (dis2<0.73) .AND. (dscrew(idz,2)<0.25)) THEN
    WRITE(21,*) '    90  (partial) dislocation'
    WRITE(21,'(a34,i3)') '     Detected core on atom number ', icore
    dis=90.d0
  ELSEIF((dis1>0.2) .AND. (dis2>0.26) .AND. (dis2<0.4) .AND. (dscrew(idz,2)>0.7)) THEN
    WRITE(21,*) '    30  (partial) dislocation'
    WRITE(21,'(a34,i3)') '     Detected core on atom number ', icore
    dis=30.d0
  ELSEIF((dis1>0.2) .AND. (dis2>0.8) .AND. (dis2<1.2) .AND. (dscrew(idz,2)>0.7)) THEN
    WRITE(21,*) '    60  (perfect) dislocation'
    WRITE(21,'(a34,i3)') '     Detected core on atom number ', icore
    dis=60.d0
  ELSE
    WRITE(21,*) '    Mixed dislocation'
    WRITE(21,'(a34,i3)') '     Detected core on atom number ', icore
    dis=dis2
  ENDIF
ELSE
  WRITE(21,*) '    No dislocation'
  dis=0.d0
ENDIF
!
DEALLOCATE(cx,yc,ir,ir1,yd,cry,crx0)
DEALLOCATE(iatom,ddep,ddep2,d)
!
!
IF((dis.NE.0) .AND. (natcmax>12)) THEN
  ! natcmax>12 prevents hazardous detection on crystal edges
  ! If core is deeper than 4 atoms from the surface, keep the plane
  ! Else displacement is not considered as a dislocation
  IF(icore>2) THEN
    WRITE(21,*) ' >>>  INFORMATIONS  WILL  BE  OUTPUT  FOR  THIS  PLANE  <<<'
    WRITE(21,'(a22,i3,a5,i3)') '          idec, idz : ', idec, ' ,   ', idz
    coremem(intplane,idz) = icore
    isintplane = 1
  ELSE
    dis = 0
  ENDIF
ELSE
  dis=0
ENDIF
IF(idz>2 .AND. coremem(intplane,idz-1)>3 .AND.                 &
  &  DABS(coremem(intplane,idz)-coremem(intplane,idz-1))>8.d0  ) THEN
  coremem(intplane,idz) = coremem(intplane,idz-1)
ELSEIF(idz>2 .AND. coremem(intplane,idz)>=REAL(NY)-1.1d0) THEN
  coremem(intplane,idz) = REAL(NY)
ENDIF
100 FORMAT(a34,i3,a14)
101 CONTINUE
!
WRITE(21,*) '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
!
! If user restrained analysis to a certain Z-layer, exit loop on idz

IF(sidz.NE.0) THEN
  nidz = 0
  pllbl = pllbl+1
  write22 = 0
  DEALLOCATE(px)
  DEALLOCATE(dedge,dscrew,dortho)
  GOTO 110
ELSEIF(sidz==0 .AND. idz<nz) THEN
  GOTO 60
ENDIF
! This ends the loop on idz
!
nidz = 0
pllbl = pllbl+1
write22 = 0
!
DEALLOCATE(px)
DEALLOCATE(dedge,dscrew,dortho)
!
! We will now compare the core position detected for each Z-plane (idz=1,nz)
! and compare it with the average core position in the {111} plane. If the
! dislocation is straight, then the core position for each Z-plane will equal
! the average one (+ or - one atom). If it is not straight, then the average
! will not match with all individual value, and variable 'corefrac' will
! have a huge value.
!
! Calculate core position average
coreav = 0.d0
corefrac = 0
DO idz=1,nz
  IF(coremem(intplane,idz)>=1) THEN
    corefrac = corefrac + NINT(coremem(intplane,idz))
  ENDIF
ENDDO
coreav = REAL(corefrac)/REAL(nz)
IF(coreav<=1.d0 .OR. corefrac<=3) THEN
  isintplane = 0
ENDIF
!
IF(isintplane==1) THEN
  WRITE(21,*) '       This {111} plane seems to contain interesting displacements'
ELSE
  WRITE(21,*) '       This {111} plane does not seem to be interesting'
ENDIF
WRITE(21,'(a34,i4)') '          Average core position : ', NINT(coreav)
!
corefrac = 0
DO idz=1,nz
  ! If dislocation core is not the same in all Z-planes, then
  ! increase corefrac
  IF(coremem(intplane,idz)<coreav-1.5d0 .OR. coremem(intplane,idz)>coreav+1.5d0) THEN
    corefrac = corefrac + 1
  ENDIF
ENDDO
!
WRITE(21,*) '            => corefrac =', corefrac
WRITE(21,*) '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
!
WRITE(*,*) ' >>>  Done parsing {111} plane number:', idec
WRITE(25,*) ' >>>  Done parsing {111} plane number:', idec
!
IF(isintplane==1) THEN
  WRITE(*,*) '          >>> IT SEEMS TO BE AN INTERESTING PLANE <<<'
  WRITE(25,*) '          >>> IT SEEMS TO BE AN INTERESTING PLANE <<<'
  IF( (recindex>NINT(nucleationtime/dtime)+200 .OR. sidz==0)                 &
    & .AND.  NOT(done22idec)                                 ) THEN
    done22idec = .TRUE.
    WRITE(22,*) ' >>>  This plane seems to be interesting:'
    WRITE(22,*) '      slopeid = ', slopeid
    WRITE(22,*) '      idec = ', idec
    IF(corefrac==0) THEN
      WRITE(22,'(a35, i3, a26)') '     containing a stacking fault and a ',  &
                               & NINT(dis), '   straight dislocation.'
    ELSEIF(corefrac<4) THEN
      WRITE(22,*) '      but only a few Z-planes actually glided, which is strange.'
      WRITE(22,*) '      This might not be a dislocation but an artefact.'
      WRITE(22,*) '      I suggest to check it for yourself.'
    ELSE
      WRITE(22,*) '      containing a stacking fault and a not straight dislocation'
      WRITE(22,*) "      (maybe a dislocation half-loop if you're lucky)"
      WRITE(22,*) ' '
      WRITE(22,*) ' (°<  WARNING: what is detected as mixed dislocation '
      WRITE(22,*) ' (°<           may involve a complex structure in this plane.'
      WRITE(22,*) ' (°<           Hence average displacements might not be relevant.'
      WRITE(22,*) ' '
      countwarning = countwarning+1
    ENDIF
    IF(recanal==1) THEN
      WRITE(22,*) ' >>>  Dislocation front detected in Z-layer idz = ', disfront
      WRITE(22,*) ' >>>  Dislocation nucleation time was estimated at', nucleationtime, ' ps'
    ENDIF
  ENDIF
ENDIF
!
102 CONTINUE
!
WRITE(21,*) ' '
WRITE(21,*) ' '
!
IF(forceoutput==1) THEN
  isintplane = 1
ENDIF
!
IF(isintplane==0) THEN
  intplane = intplane-1
ENDIF
!
! If user restrained analysis to a specific {111} plane, exit loop on idec
IF(sidec.NE.0) THEN
  GOTO 110
ELSEIF((sidec==0).AND.(idec<(NINT(x0/a0)+addyr*ny/2))) THEN
  GOTO 50
ENDIF
! Ends the loop on idec
!
IF(slopeid==1) THEN
	WRITE(*,*) ' >>>  PARSE, PART 1 : DONE'
	WRITE(*,*) ' '
	WRITE(25,*) ' >>>  PARSE, PART 1 : DONE'
	WRITE(25,*) ' '
ELSE
	WRITE(*,*) ' >>>  PARSE, PART 2 : DONE'
	WRITE(*,*) ' '
	WRITE(25,*) ' >>>  PARSE, PART 2 : DONE'
	WRITE(25,*) ' '
ENDIF
!
! If user restrained analysis to a certain slope, exit loop on slopeid
IF(sslopeid .NE. 0) THEN
  GOTO 110
ELSEIF((sslopeid==0) .AND. (slopeid<2)) THEN
  GOTO 40
ENDIF
! This ends loop on slopeid
!
110 CONTINUE
intplaneup = intplane
!
WRITE(21,*) '******************************************'
WRITE(21,*) ' >>>  FOUND ', intplaneup, ' INTERESTING PLANES.'
WRITE(25,*) ' >>>  FOUND ', intplaneup, ' INTERESTING PLANES.'

111 FORMAT (a6,i3,a21)
!
! ## OUTPUT EDGE AND SCREW COMPONENTS INTO FILES
! ###############################################
! NB: Following files are directly readable by XmGrace
!
! Individual values of displacements will be written in a file for
! each atom in an interesting {111} plane. For convenience, we will
! output all the displacements for a given plane in a separate file,
! sothat there are as many output files as interesting planes.
! Also, average values of displacements will be computed and output
! in separate files. 
!
IF(intplaneup==0) THEN
  WRITE(21,*) ' >>>  No dislocation found => nothing written in .dat files.'
  WRITE(*,*) ' >>>  No interesting plane found'
  GOTO 500
ENDIF
!
ALLOCATE(ddedge(20,nz,natpmax))
ALLOCATE(corewidth(20,nz), gpcorewidth(20,nz))
!
ALLOCATE(dedgeavg(natcup))
ALLOCATE(dscrewavg(natcup))
DO intplindex=1, intplaneup
!  
  dedgeavg(:) = 0.d0
  dscrewavg(:) = 0.d0
  corewidth(:,:) = 0.d0
  gpcorewidth(:,:) = 0.d0
  ddedge(:,:,:) = 0.d0
!
  IF(recanal==0) THEN
    IF(sidz==0) THEN
      WRITE(ich3,'(i3)') intplindex
      ich3 = ADJUSTL(ich3)
      sdatfile = 'screw_allZ_'//TRIM(ich3)//'.dat'
      edatfile = 'edge_allZ_'//TRIM(ich3)//'.dat'
      savgfile = 'screw_avg_'//TRIM(ich3)//'.dat'
      eavgfile = 'edge_avg_'//TRIM(ich3)//'.dat'
      odatfile = 'ortho_allZ_'//TRIM(ich3)//'.dat'
      coredatfile = 'core_'//TRIM(ich3)//'.dat'
      corewidthdatfile = 'corewidth_'//TRIM(ich3)//'.dat'
      ddedgefile = 'ddedge_'//TRIM(ich3)//'.dat'
    ELSE
      WRITE(ich3,'(i3)') idz
      ich3 = ADJUSTL(ich3)
      sdatfile = 'screw_idz'//TRIM(ich3)//'.dat'
      edatfile = 'edge_idz'//TRIM(ich3)//'.dat'
      ddedgefile = 'ddedge_idz'//TRIM(ich3)//'.dat'
      odatfile = 'ortho_idz'//TRIM(ich3)//'.dat'
    ENDIF
  ELSE ! ie, if recanal.NE.0
    IF(sidz==0) THEN
      WRITE(ich2,'(i2)') intplindex
      WRITE(rich5,'(i5)') recindex
      ich2 = ADJUSTL(ich2)
      rich5 = ADJUSTL(rich5)
      sdatfile = 'screw_allZ_'//TRIM(ich2)//'_t'//TRIM(rich5)//'.dat'
      edatfile = 'edge_allZ_'//TRIM(ich2)//'_t'//TRIM(rich5)//'.dat'
      savgfile = 'screw_avg_'//TRIM(ich2)//'_t'//TRIM(rich5)//'.dat'
      eavgfile = 'edge_avg_'//TRIM(ich2)//'_t'//TRIM(rich5)//'.dat'
      odatfile = 'ortho_allZ_'//TRIM(ich2)//'_t'//TRIM(rich5)//'.dat'
      coredatfile = 'core_'//TRIM(ich2)//'_t'//TRIM(rich5)//'.dat'
      corewidthdatfile = 'corewidth_'//TRIM(ich2)//'_t'//TRIM(rich5)//'.dat'
      ddedgefile = 'ddedge_'//TRIM(ich2)//'_t'//TRIM(rich5)//'.dat'
    ELSE ! ie, if sidz.NE.0
      WRITE(ich3,'(i3)') sidz
      WRITE(rich5,'(i5)') recindex
      ich3 = ADJUSTL(ich3)
      rich5 = ADJUSTL(rich5)
      sdatfile = 'screw_idz_'//TRIM(ich3)//'_t'//TRIM(rich5)//'.dat'
      edatfile = 'edge_idz_'//TRIM(ich3)//'_t'//TRIM(rich5)//'.dat'
      ddedgefile = 'ddedge_idz_'//TRIM(ich3)//'_t'//TRIM(rich5)//'.dat'
      odatfile = 'ortho_idz_'//TRIM(ich3)//'_t'//TRIM(rich5)//'.dat'
    ENDIF
  ENDIF ! recanal
!
  OPEN(UNIT=16,FILE=edatfile,FORM='FORMATTED',STATUS='UNKNOWN')
  OPEN(UNIT=17,FILE=sdatfile,FORM='FORMATTED',STATUS='UNKNOWN')
  OPEN(UNIT=18,FILE=odatfile,FORM='FORMATTED',STATUS='UNKNOWN')
  IF(sidz==0) THEN
    OPEN(UNIT=23,FILE=eavgfile,FORM='FORMATTED',STATUS='UNKNOWN')
    OPEN(UNIT=24,FILE=savgfile,FORM='FORMATTED',STATUS='UNKNOWN')
    OPEN(UNIT=29,FILE=coredatfile,FORM='FORMATTED',STATUS='UNKNOWN')
    OPEN(UNIT=52,FILE=corewidthdatfile,FORM='FORMATTED',STATUS='UNKNOWN')
  ENDIF
!
  IF(sidz==0) THEN
    DO i=1,natcup-1  ! Use natcup-1 to avoid artefact "last value=0 or infinite"
                     ! (idem for intplaneup)
      ! For a given {111} plane, we calculate the average displacements
      ! of each atom label, from a surface (i=1) to the other (i=natcup)
      DO idz=1, nz
        dedgeavg(i) = dedgeavg(i) + dedgefinal(intplindex,idz,i)
        dscrewavg(i) = dscrewavg(i) + dscrewfinal(intplindex,idz,i)
      ENDDO
      dedgeavg(i) = dedgeavg(i) /nz     ! Average values for
      dscrewavg(i) = dscrewavg(i) /nz   ! atoms labeled i
    ENDDO
  ENDIF
!
  DO i=1,natcup-1
    IF(sidz==0) THEN
      ! Write average values
      WRITE(23,150) i, dedgeavg(i)
      WRITE(24,150) i, dscrewavg(i)
      ! Write all individual values
      WRITE(17,150) i, (dscrewfinal(intplindex,idz,i), idz=1,nz)
      WRITE(16,150) i, (dedgefinal(intplindex,idz,i), idz=1,nz)
      WRITE(18,150) i, (dorthofinal(intplindex,idz,i), idz=1,nz)
    ELSE
      ! Write all individual values
      WRITE(17,150) i, dscrewfinal(intplindex,sidz,i)
      WRITE(16,150) i, dedgefinal(intplindex,sidz,i)
      WRITE(18,150) i, dorthofinal(intplindex,sidz,i)
    ENDIF
  ENDDO
!
  CLOSE(16)
  WRITE(21,*) ' >>>  WROTE FILE ', edatfile
  WRITE(25,*) ' >>>  WROTE FILE ', edatfile
  WRITE(*,*) ' >>>  WROTE FILE ', edatfile
  CLOSE(17)
  WRITE(21,*) ' >>>  WROTE FILE ', sdatfile
  WRITE(25,*) ' >>>  WROTE FILE ', sdatfile
  WRITE(*,*) ' >>>  WROTE FILE ', sdatfile
  CLOSE(18)
  WRITE(21,*) ' >>>  WROTE FILE ', odatfile
  WRITE(25,*) ' >>>  WROTE FILE ', odatfile
  WRITE(*,*) ' >>>  WROTE FILE ', odatfile
  IF(sidz==0) THEN
    CLOSE(23)
    WRITE(21,*) ' >>>  WROTE FILE ', eavgfile
    WRITE(25,*) ' >>>  WROTE FILE ', eavgfile
    WRITE(*,*) ' >>>  WROTE FILE ', eavgfile
    CLOSE(24)
    WRITE(21,*) ' >>>  WROTE FILE ', savgfile
    WRITE(25,*) ' >>>  WROTE FILE ', savgfile
    WRITE(*,*) ' >>>  WROTE FILE ', savgfile
  ENDIF
  !
  ! Calculate the derivative of edge displacements
  WRITE(21,*) ' >>>  Calculate derivative of edge displacements and core width'
  DO idz=1,nz
    ddedge(intplindex,idz,1) = 0.d0
    ddedge(intplindex,idz,2) =                                    &
     &   ( dedgefinal(intplindex,idz,3) -                       &
     &     dedgefinal(intplindex,idz,1)     ) /2.d0
    DO i=3,natcup-1
      ddedge(intplindex,idz,i) =                                  &
       & ( ( ( dedgefinal(intplindex,idz,i+1) -                   &
       &       dedgefinal(intplindex,idz,i-1)  ) /2.d0 )          &
       &  +( ( dedgefinal(intplindex,idz,i+2) -                   &
       &       dedgefinal(intplindex,idz,i-2)  ) /4.d0 )  ) /2.d0
      IF(ddedge(intplindex,idz,i)>=ddedge(intplindex,idz,i-1)) THEN
        maxddedge = ddedge(intplindex,idz,i)
      ENDIF
    ENDDO
    ! Extract the dislocation core width
    DO i=1,natcup-1
      IF(ddedge(intplindex,idz,i)>maxddedge/2.d0   .AND. &
       & ddedge(intplindex,idz,i-1)<maxddedge/2.d0 .AND. &
       & corewidth1==0.d0 ) THEN
        corewidth1 = ddedge(intplindex,idz,i)
      ELSEIF(ddedge(intplindex,idz,i)<maxddedge/2.d0 .AND. &
       & ddedge(intplindex,idz,i+1)>maxddedge/2.d0   .AND. &
       & corewidth2==0.d0 ) THEN
        corewidth2 = ddedge(intplindex,idz,i)
      ENDIF
    ENDDO
    corewidth(intplindex,idz) = corewidth2-corewidth1
  ENDDO
  OPEN(UNIT=26,FILE=ddedgefile,FORM='FORMATTED',STATUS='UNKNOWN')
  DO i=1,natcup-1
    IF(sidz==0) THEN
      WRITE(26,150) i, (ddedge(intplindex,idz,i),idz=1,nz)
    ELSE
      WRITE(26,150) i, ddedge(intplindex,sidz,i)
    ENDIF
  ENDDO
  CLOSE(26)
  WRITE(21,*) ' >>>  WROTE FILE ', ddedgefile
  WRITE(25,*) ' >>>  WROTE FILE ', ddedgefile
  WRITE(*,*) ' >>>  WROTE FILE ', ddedgefile
  !
  ! If gnuplot is available, use it to determine more precisely
  ! the position and width of the dislocation core
  IF(gnuplot==1) THEN
    fita = 1.d0
    fitb = 1.d0
    fitc = -1.d0
    fitd = -1.d0
    IF(sidz==0) THEN
      DO idz=2,nz
        gnuplotfit = 0.d0
        !If the program already detected there was no dislocation, or
        !the dislocation has crossed the crystal, its useless to use gnuplot
        IF(coremem(intplindex,idz)/=0.d0 .AND. coremem(intplindex,idz)/=REAL(NY)) THEN
          !Set the edge displacements file name (has to be the same as above)
          IF(recanal==0) THEN
            WRITE(ich3,'(i3)') intplindex
            ich3 = ADJUSTL(ich3)
            edatfile = 'edge_allZ_'//TRIM(ich3)//'.dat'
            efitfile = 'edgefit_allZ_'//TRIM(ich3)//'.dat'
          ELSE ! ie, if recanal.NE.0
            WRITE(ich2,'(i2)') intplindex
            WRITE(rich5,'(i5)') recindex
            ich2 = ADJUSTL(ich2)
            rich5 = ADJUSTL(rich5)
            edatfile = 'edge_allZ_'//TRIM(ich2)//'_t'//TRIM(rich5)//'.dat'
            efitfile = 'edgefit_allZ_'//TRIM(ich2)//'_t'//TRIM(rich5)//'.dat'
          ENDIF
          WRITE(ich3,'(i3)') idz
          ich3 = ADJUSTL(ich3)
          !Build the input file for gnuplot
          OPEN(UNIT=31,FILE='dechar_111.gp',STATUS='UNKNOWN')
          WRITE(31,*) "set print 'dechar_111.tmp'"
          WRITE(31,'(a2,e12.6)') 'a=', fita
          WRITE(31,'(a2,e12.6)') 'b=', fitb
          WRITE(31,'(a2,e12.6)') 'c=', fitc
          WRITE(31,'(a2,e12.6)') 'd=', fitd
          WRITE(31,*) 'FIT_LIMIT = 1.e-7'
          WRITE(31,*) 'FIT_MAXITER = 6000'
          WRITE(31,*) 'f(x)=a*atan(b*x+c)+d'
          WRITE(31,*) "fit f(x) '"//TRIM(edatfile)//"' using 1:"//TRIM(ich3)//" via a,b,c,d"
          WRITE(31,*) "set print 'dechar_111.fit'"
          WRITE(31,*) 'print a'
          WRITE(31,*) 'print b'
          WRITE(31,*) 'print c'
          WRITE(31,*) 'print d'
          CLOSE(31)
          !Run fit
          IF(idz==nz .OR. idz==sidz) THEN
            WRITE(21,*) ' >>>  Start curve fitting, calling GnuPlot...'
            WRITE(*,*) ' >>>  Start curve fitting, calling GnuPlot...'
          ENDIF
          CALL SYSTEM('gnuplot dechar_111.gp')
          !Read values obtained from the fit
          OPEN(UNIT=32,FILE='dechar_111.fit',STATUS='UNKNOWN')
          READ(32,*) fita
          READ(32,*) fitb
          READ(32,*) fitc
          READ(32,*) fitd
          CLOSE(UNIT=32)
          !Calculate the core position
          gnuplotfit = -fitc/fitb
          !Controllers to avoid dummy core detection at +/-infinity
          !(happens when there is no dislocation and the fit algorithm diverges)
          !and to smooth the shape of dislocation line (=avoid too sharp changes)
          IF(gnuplotfit<0.d0) THEN
            gnuplotfit=0.d0
          ELSEIF(gnuplotfit>REAL(NY) .OR.                                      &
            &    (coremem(intplindex,idz-1)>REAL(NY)-5 .AND.                   &
            &     gnuplotfit<REAL(NY)/2.d0                   )  ) THEN
            coremem(intplindex,idz)=REAL(NY)
          ELSEIF(coremem(intplindex,idz-1)>REAL(NY)-10 .AND.                   &
            &    gnuplotfit<2.d0*REAL(NY)/3.d0                  ) THEN
            gnuplotfit=REAL(NY)
          ELSEIF(idz>2 .AND. gnuplotfit>coremem(intplindex,idz-1)              &
            & .AND. coremem(intplindex,idz-1)<=REAL(NY)/4.d0 .AND. idz>2 .AND. &
            & (coremem(intplindex,idz-1)/=0.d0 .AND.                           &
            &  gnuplotfit/coremem(intplindex,idz-1)>=4.d0  )  .OR.             &
            & gnuplotfit-coremem(intplindex,idz-1)>=8.d0                        ) THEN
            gnuplotfit = coremem(intplindex,idz-1)
          ELSEIF(idz>2 .AND. gnuplotfit<coremem(intplindex,idz-1)              &
            & .AND. coremem(intplindex,idz-1)<=REAL(NY)/4.d0 .AND. idz>2 .AND. &
            & (gnuplotfit/=0.d0 .AND.                                          &
            &  coremem(intplindex,idz-1)/gnuplotfit>=4.d0 )      .OR.          &
            & coremem(intplindex,idz-1)-gnuplotfit>=8.d0           ) THEN
            gnuplotfit = coremem(intplindex,idz-1)
          ENDIF
          ! Keep the fitted value only if the difference with previous value
          ! is not too big, ie. below five atoms (arbitrary)
          ! (this is mainly to avoid dummy fits)
          IF( DABS(gnuplotfit-coremem(intplindex,idz)) <= 5.d0 )  THEN
            coremem(intplindex,idz) = gnuplotfit
          ENDIF
          ! Delete temporary files used during the fit
          CALL SYSTEM('rm -f fit.log')
          IF(writelog==0) THEN
            CALL SYSTEM('rm -f dechar_111.gp dechar_111.fit dechar_111.tmp fit.log')
          ENDIF
          IF(idz==1 .OR. idz==sidz) THEN
            WRITE(21,*) ' >>>  Curve fitting complete.'
            WRITE(21,*) ' '
            WRITE(*,*) ' >>>  Curve fitting complete.'
          ENDIF
          ! Calculate the extension of the dislocation core
          ! = Full width at half maximum of the derivative of
          ! the displacements :
          !             df/dx = (a*b) / (1+(bx+c))
          !!!  TO BE IMPLEMENTED IN THIS VERSION  !!!
          IF(idz==1 .OR. idz==sidz) THEN
            WRITE(21,*) ' >>> Calculate dislocation core width from fit'
          ENDIF
          IF(fitb.NE.0.d0) gpcorewidth(intplindex,idz) = 2/fitb
          IF(gpcorewidth(intplindex,idz)<0.d0  .OR.                            &
           & gpcorewidth(intplindex,idz)>20.d0     ) THEN
            gpcorewidth(intplindex,idz)=0.d0
          ELSE
            corewidth(intplindex,idz)=gpcorewidth(intplindex,idz)
          ENDIF
        ENDIF
        WRITE(21,*) '      ', idz, corewidth(intplindex,idz)
        WRITE(52,*) idz, corewidth(intplindex,idz)
      ENDDO !loop on idz
    ENDIF   !if(sidz==0)
  ENDIF   !if(gnuplot==1)
  ! Make sure the periodic boundary conditions are respected
  ! (no matter how core position was calculated)
  coremem(intplindex,1) = coremem(intplindex,nz)
  !
  150 FORMAT(i4,100(1X,e16.8))
  ! NB: GnuPlot does not read double-precision numbers (1.5d-1),
  !     only scientific notations (1.5e-1)
  !
  ! Output dislocation core position in a [.dat] file
  IF(sidz==0 .AND. isintplane==1) THEN
    WRITE(21,*) ' >>>  Calculating dislocation core position'
    ! Distances will be in nm (a0 was in A); orthogonally to the step,
    ! distance between two atoms is a0*sqrt(6)/4; along the step, distance
    ! between two atoms is a0*sqrt(2)/2
    IF(recanal==1) THEN
      vcount = (recindex-rnstart)/rninc + 1
      DPortho(vcount) = coremem(intplindex,disfront)*a0*DSQRT(6.d0)/40.d0
      WRITE(21,*) ' >>>  Orthogonal to the step, position=', DPortho(vcount)
    ENDIF
    DO idz=1, nz
      WRITE(29,*) idz, coremem(intplindex,idz)
      IF(recanal==1) THEN
        ! Determine position of dislocation core along the step
        ! this is to plot dislocation velocities as a function
        ! of time in these two directions
        idzvm5 = idz-5
        idzvm2 = idz-2
        idzvm1 = idz-1
        idzvp1 = idz+1
        idzvp2 = idz+2
        idzvp5 = idz+5
        IF(idz<=5) idzvm5 = idz+nz-5
        IF(idz<=2) idzvm2 = idz+nz-2
        IF(idz==1) idzvm1 = idz+nz-1
        IF(idz>=nz)   idzvp1 = idz-nz+1
        IF(idz>=nz-2) idzvp2 = idz-nz+2
        IF(idz==nz-5) idzvp5 = idz-nz+5
        IF(coremem(intplindex,idzvm1)==0 .AND. coremem(intplindex,idz)>0      &
          & .AND.  coremem(intplindex,idzvm2)==0 ) THEN
          DPparaL(vcount) = REAL(idzvm1)*(a0/10.d0)*DSQRT(2.d0)/2.d0
        ELSEIF(coremem(intplindex,idzvm1)==0 .AND. coremem(intplindex,idz)==0   &
              & .AND. coremem(intplindex,idzvm2)>0 ) THEN
          DPparaR(vcount) = REAL(idzvm1)*(a0/10.d0)*DSQRT(2.d0)/2.d0
        ENDIF
      ENDIF
    ENDDO
    IF(recanal==1) THEN
      WRITE(21,*) ' >>>  Along the step, positions='
      WRITE(21,*) '      ', DPparaL(vcount), DPparaR(vcount)
    ENDIF
    CLOSE(29)
    WRITE(21,*) ' '
    WRITE(21,*) ' >>>  WROTE FILE ', coredatfile
    WRITE(25,*) ' >>>  WROTE FILE ', coredatfile
    WRITE(*,*) ' >>>  WROTE FILE ', coredatfile
    IF(recanal.NE.0)  corewidthT(vcount) = corewidth(intplindex,disfront)
  ENDIF
  WRITE(21,*) ' '
  !
ENDDO  ! Loop on intplindex
!
DEALLOCATE(dedgeavg,dscrewavg)
DEALLOCATE(ddedge)
DEALLOCATE(corewidth,gpcorewidth)
!
500 CONTINUE
DEALLOCATE(dedgefinal, dscrewfinal,dorthofinal)
DEALLOCATE(coremem)
!
DEALLOCATE(coordR, coordNR)
! If recursive analysis is set, restart analysis with next file
IF(recanal.NE.0) THEN
  WRITE(21,*) ' '
  WRITE(21,*) ' '
  nzcfgcount = 0  
  donefinal = 0
  isintplane = 0
  intplane = 1
  intplaneup = 0
  corefrac = 0
  vcountmax = vcount
  recindexmax = recindex
  IF(recindex<100000 .AND. recindex<rnend) THEN
    recindex = recindex+rninc
    slopeid = sslopeid
    idec = sidec
    idz = sidz
    GOTO 20
  ENDIF
ENDIF
!
!
600 CONTINUE
IF(reccount==0) THEN
  WRITE(21,*) ' '
  WRITE(21,*) ' (°<  WARNING: No file was found. Check the file names you'
  WRITE(21,*) ' (°<           specified in the  dechar_111.in file.'
  WRITE(22,*) ' '
  WRITE(22,*) ' (°<  WARNING: No file was found. Check the file names you'
  WRITE(22,*) ' (°<           specified in the  dechar_111.in file.'
  WRITE(*,*) ' '
  WRITE(*,*) ' (°<  WARNING: No file was found. Check the file names you'
  WRITE(*,*) ' (°<           specified in the dechar_111.in  file.'
  GOTO 1000
ENDIF
!
! Calculate and write dislocation speed
IF(recanal==1 .AND. sidz==0) THEN
  ALLOCATE(Vortho(vcountmax+1), Vpara(vcountmax+1))
  Vortho(:) = 0.d0
  Vpara(:) = 0.d0
  VorthoT = 'Vortho_f_time.dat'
  VparaT = 'Vpara_f_time.dat'
  VorthoS = 'Vortho_f_Size.dat'
  CwidthT = 'CoreWidth_f_Size.dat'
  OPEN(UNIT=32,FILE=VorthoT,FORM='FORMATTED',STATUS='UNKNOWN')
  OPEN(UNIT=33,FILE=VparaT,FORM='FORMATTED',STATUS='UNKNOWN')
  OPEN(UNIT=34,FILE=VorthoS,FORM='FORMATTED',STATUS='UNKNOWN')
  OPEN(UNIT=53,FILE=CwidthT,FORM='FORMATTED',STATUS='UNKNOWN')
  !Speeds will be in nm/ps = km/s
  !and plotted versus time (origin of time=nucleation time)
  WRITE(21,*) ' '
  WRITE(21,*) ' >>> Calculate dislocation speed'
  WRITE(*,*) ' '
  WRITE(*,*) ' >>> Calculate dislocation speed'
  DO vcount=1, vcountmax-1
    IF(DPortho(vcount+1)==0.d0 .AND. DPortho(vcount)==0.d0) THEN
      Vortho(Vcount) = 0.d0
    ELSE
      Vortho(vcount) = ( DPortho(vcount+1)-DPortho(vcount) )  &
                     &   /(REAL(rninc)*dtime)
    ENDIF
    IF(Vortho(vcount) >10.d0) THEN
      Vortho(vcount) = 0.d0
      WRITE(21,*) ' (°<  WARNING: Dislocation speed is too high and exceeds'
      WRITE(21,*) '               sound speed. Check if the dislocation core'
      WRITE(21,*) '               is correctly detected.'
    ENDIF
    !
    IF( DPparaL(vcount)<=2 .OR. DPparaR(vcount)<=2   ) THEN
      Vpara(Vcount) = 0.d0
    ELSEIF( DPparaL(vcount+1)==DPparaL(vcount) .OR.           &
            DPparaR(vcount+1)==DPparaR(vcount)       ) THEN
      Vpara(Vcount) = 0.d0
    ELSE
      Vpara(vcount) = ( DPparaL(vcount)-DPparaL(vcount+1)     &
                    &  +DPparaR(vcount+1)-DPparaR(vcount) ) /(2.d0*REAL(rninc)*dtime)
    ENDIF
    IF(Vpara(vcount) >10.d0) THEN
      Vpara(vcount) = 0.d0
      WRITE(21,*) ' (°<  WARNING: Dislocation speed is too high and exceeds'
      WRITE(21,*) '               sound speed. Check if the dislocation core'
      WRITE(21,*) '               is correctly detected.'
    ENDIF
    WRITE(21,*) '     Vortho (', vcount, ') = ', Vortho(vcount)
    WRITE(21,*) '     Vpara (', vcount, ' ) = ', Vpara(vcount)
    IF(Vortho(vcount)>=0.001d0) THEN
      WRITE(32,*) (vcount*rninc+rnstart)*dtime-nucleationtime, Vortho(vcount)
      WRITE(34,*) DPortho(vcount)*bedge*2.d0, Vortho(vcount)
    ENDIF
!    IF(Vpara(vcount)>=0.001d0) THEN
!      WRITE(33,*) (vcount*rninc+rnstart)*dtime-nucleationtime, Vpara(vcount)
!    ENDIF
    WRITE(53,*) DPortho(vcount)*bedge*2.d0, corewidthT(vcount)
  ENDDO
  CLOSE(34)
  CLOSE(33)
  CLOSE(32)
  CLOSE(53)
  WRITE(*,*) ' '
  WRITE(21,*) ' '
  WRITE(25,*) ' '
  WRITE(*,*) ' >>>  WROTE FILE ', VorthoT
  WRITE(21,*) ' >>>  WROTE FILE ', VorthoT
  WRITE(25,*) ' >>>  WROTE FILE ', VorthoT
  WRITE(*,*) ' >>>  WROTE FILE ', VparaT
  WRITE(21,*) ' >>>  WROTE FILE ', VparaT
  WRITE(25,*) ' >>>  WROTE FILE ', VparaT
  WRITE(*,*) ' >>>  WROTE FILE ', VorthoS
  WRITE(21,*) ' >>>  WROTE FILE ', VorthoS
  WRITE(25,*) ' >>>  WROTE FILE ', VorthoS
  WRITE(*,*) ' (°<  WARNING: dislocation speeds are relevant only when analyzing'
  WRITE(*,*) '               files coming from a dynamic simulation.'
  WRITE(21,*) ' (°<  WARNING: dislocation speeds are relevant only when analyzing'
  WRITE(21,*) '               files coming from a dynamic simulation.'
  WRITE(25,*) ' (°<  WARNING: dislocation speeds are relevant only when analyzing'
  WRITE(25,*) '               files coming from a dynamic simulation.'
  WRITE(*,*) ' >>>  WROTE FILE ', CwidthT
  WRITE(21,*) ' >>>  WROTE FILE ', CwidthT
  WRITE(25,*) ' >>>  WROTE FILE ', CwidthT
  DEALLOCATE(Vortho,Vpara,corewidthT)
ENDIF
!
! Delete all files user does not want
IF(outedgef==0) CALL SYSTEM('rm -f edge*')
IF(outscrewf==0) CALL SYSTEM('rm -f screw*')
IF(outorthof==0) CALL SYSTEM('rm -f ortho*')
IF(outcoreposf==0) CALL SYSTEM('rm -f core_*')
IF(outcorewidthf==0) CALL SYSTEM('rm -f corewidth*')
IF(outvelocityf==0) CALL SYSTEM('rm -f V*.dat')
!
!
WRITE(*,*) ' '
WRITE(*,*) ' >>>  Bravo ! Program ran successfully  ^_^ '
!
!
! ###############################################
!
IF(warning==1) THEN
  WRITE(21,*) ' '
  WRITE(21,*) ' (°<  WARNING : Huge displacements were detected while parsing'
  WRITE(21,*) ' (°<            the crystal. Either your crystal is extremely'
  WRITE(21,*) ' (°<            deformed and program output may not be accurate,'
  WRITE(21,*) ' (°<            either there is a mismatch between the two systems'
  WRITE(21,*) ' (°<            you are trying to compare (eg. atoms may not be'
  WRITE(21,*) ' (°<            sorted in the same way in position files).'
  WRITE(22,*) ' '
  WRITE(22,*) ' (°<  WARNING : Huge displacements were detected while parsing'
  WRITE(22,*) ' (°<            the crystal. Either your crystal is extremely'
  WRITE(22,*) ' (°<            deformed and program output may not be accurate,'
  WRITE(22,*) ' (°<            either there is a mismatch between the two systems'
  WRITE(22,*) ' (°<            you are trying to compare (eg. atoms may not be'
  WRITE(22,*) ' (°<            sorted in the same way in position files).'
  WRITE(*,*) ' (°<  WARNING : Huge displacements were detected while parsing'
  WRITE(*,*) ' (°<            the crystal. Either your crystal is extremely'
  WRITE(*,*) ' (°<            deformed and program output may not be accurate,'
  WRITE(*,*) ' (°<            either there is a mismatch between the two systems'
  WRITE(*,*) ' (°<            you are trying to compare (eg. atoms may not be'
  WRITE(*,*) ' (°<            sorted in the same way in position files).'
  countwarning = countwarning+1
ENDIF
!
IF(warning2==1) THEN
  WRITE(21,*) ' '
  WRITE(21,*) ' (°<  WARNING : Huge displacements orthogonal to {111} planes'
  WRITE(21,*) ' (°<            were detected. This program is not designed to'
  WRITE(21,*) ' (°<            deal with this kind of behaviour, hence output'
  WRITE(21,*) ' (°<            may not be accurate.'
  WRITE(22,*) ' '
  WRITE(22,*) ' (°<  WARNING : Huge displacements orthogonal to {111} planes'
  WRITE(22,*) ' (°<            were detected. This program is not designed to'
  WRITE(22,*) ' (°<            deal with this kind of behaviour, hence output'
  WRITE(22,*) ' (°<            may not be accurate.'
  WRITE(*,*) ' (°<  WARNING : Huge displacements orthogonal to {111} planes'
  WRITE(*,*) ' (°<            were detected. This program is not designed to'
  WRITE(*,*) ' (°<            deal with this kind of behaviour, hence output'
  WRITE(*,*) ' (°<            may not be accurate.'
  countwarning = countwarning+1
ENDIF
!
WRITE(21,*) ' '
WRITE(21,*) ' '
WRITE(21,'(a44,a20)') ' >>>  Main results have been output in file ',   &
     &                 ADJUSTL(TRIM(outfile))
WRITE(*,'(a44,a20)') ' >>>  Main results have been output in file ',              &
     &                 ADJUSTL(TRIM(outfile))
CLOSE(25)
!
1000 CONTINUE
!
WRITE(22,*) ' '
WRITE(22,*) ' >>>  ANALYZED FILES:', reccount
WRITE(22,*) ' >>>  ERRORS:        ', counterror
WRITE(22,*) ' >>>  WARNINGS:      ', countwarning
CLOSE(22)
!
1100 CONTINUE
WRITE(21,*) ' '
WRITE(21,*) ' >>>  ANALYZED FILES:', reccount
WRITE(21,*) ' >>>  ERRORS:        ', counterror
WRITE(21,*) ' >>>  WARNINGS:      ', countwarning
WRITE(21,*) ' '
WRITE(21,*) '     (`._.``._.- END OF PROGRAM -._.``._.`)'
CLOSE(21)
WRITE(*,*) ' '
IF(recanal==1) THEN
  WRITE(*,*) ' >>>  ANALYZED FILES:', reccount
ENDIF
WRITE(*,*) ' >>>  ERRORS:        ', counterror
WRITE(*,*) ' >>>  WARNINGS:      ', countwarning
WRITE(*,*) ' '
WRITE(*,*) '         (`._.``._.- END OF PROGRAM -._.``._.`)'
WRITE(*,*) ' '
!
!
IF(writelog==0) CALL SYSTEM('rm -f dechar_111.log')
!
!##########################################################
!
END PROGRAM dechar_111
!
!##########################################################
!###################### END OF FILE #######################
!##########################################################
!
